'''

:author: fv
:date: Created on 12 mai 2021
'''

from PyQt5.Qt import QFrame, QLabel, QHBoxLayout, QSizePolicy

class QRegister(QFrame):
    '''
    classdocs
    '''


    def __init__(self, parent):
        '''
        Constructor
        '''
        QFrame.__init__(self, parent)
        self._reg = None
        self.setContentsMargins(0,0,0,0)
        self._layout = QHBoxLayout(self)
        self._layout.setContentsMargins(0,0,0,0)
        self._labelText = QLabel(self)
        self._labelText.setObjectName('labelText')
        self._labelText.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        self._layout.addWidget(self._labelText)
        
        self._labelValue = QLabel(self)
        self._labelValue.setObjectName('labelValue') 
        self._labelValue.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        self._layout.addWidget(self._labelValue)
        
        
        self.setLayout(self._layout)
        self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Fixed)
        
    def onRegisterChange(self, reg, old_value=None, new_value=None):
        if new_value is None:
            new_value = self._reg.value
        self._labelValue.setText( '{0:d} (0x{0:04x})'.format(new_value) )
        
    
        
    def setReg(self, reg=None ):
        if self._reg is not None:
            self._reg.disconnect(self.onRegisterChange)  
        self._labelValue.clear()
        self._reg = reg
        if self._reg is not None:
            self._labelText.setText(self._reg.name)
            self.setToolTip(self._reg.descr)
            self._reg.connect(self.onRegisterChange)
            self.onRegisterChange(self._reg, self._reg.value, self._reg.value)
        else:
            self._labelText.setText('~')
            self.setToolTip('')
        
    @property
    def reg(self):
        return self._reg
    
    


'''

:author: fv
:date: Created on 28 mars 2021
'''
import lxml.etree as ET
from PyQt5.QtCore import QObject, pyqtSignal
from PyQt5.Qt import  Qt, QGraphicsSvgItem, QGraphicsView, QGraphicsScene, QSvgRenderer,\
    QTimer, QPointF, QRectF, QByteArray
import logging

KEY_REPEAT_PERIOD = 200 #ms

KEY_FIRST_REPEAT_COUNT = 2

KEY_LONG_COUNT = 3 # nb repeates


def drawText(painter, p, flags, text):
    size = 32767.0
    corner = QPointF(p.x(), p.y() - size)
    if flags & Qt.AlignHCenter:
        corner.setX(corner.x() - size/2.0)
    elif flags & Qt.AlignRight:
        corner.setX(corner.x() - size)
    if flags & Qt.AlignVCenter:
        corner.setY(corner.y() +size/2.0)
    elif flags & Qt.AlignTop:
        corner.setY(corner.y() + size)
    else:
        flags |= Qt.AlignBottom
    rect = QRectF(corner.x(), corner.y(), size, size)
    return painter.drawText(rect, flags, text)
    



class SvgItem(QGraphicsSvgItem):
    '''Base class for SVG item
    
    :param str oid: The item ID
    :param QSvgPanel owner: The item's owner panel
    :param QWidget parent: optionally the parent widget
    '''
    def __init__(self, svg, owner, renderer, parent=None):
        super().__init__(parent)
        self.svg = svg
        xStyle = svg.get('style')
        if xStyle is not None :
            try:
                self.style = dict( ( kv.split(':') for kv in xStyle.split(';') ) )
            except KeyError:
                self.style = None
        else:
            self.style = None
        self._owner = owner
        self.setSharedRenderer(renderer)
        self.setElementId(svg.get('id'))
        self._bounds = renderer.boundsOnElement(self.elementId())
        self.setPos(self._bounds.topLeft())
        self._owner.log.debug(self.__class__.__name__+":"+self.elementId()+":Initialized")
        #self.setFlag(QGraphicsItem.ItemIsSelectable, True) #horrible selection-box
        
    def initialize(self):
        pass
        
    def _update(self):
        self.update()
    
    def changeAttribute(self, name, value):
        pass
    
    @property
    def owner(self):
        return self._owner
    
    @property
    def board(self):
        return self._owner._board
    
    @property
    def id(self):
        return self.elementId()
    @property
    def bounds(self):
        return self._bounds
    
class SvgLayout(SvgItem):
    '''Class for the layout (draw) of panel.'''
    pass    

class SvgSensitiveItem(SvgItem):
    '''Base class for sensitive items'''
    
    mousePressed = pyqtSignal(QObject, name='mousePressed')
    '''Signal for *mouse pressed* event'''
     
    mouseReleased = pyqtSignal(QObject, name='mouseReleased')
    '''Signal for *mouse released* event'''
    
    
    def mousePressEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        '''Rewrite of the handler for the "mouse pressed" event'''
        self.mousePressed.emit(self)
        super().mousePressEvent(event)
        event.accept()

    def mouseReleaseEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        '''Rewrite of the handler for the "mouse released" event'''
        self.mouseReleased.emit(self)
        super().mouseReleaseEvent(event)

class SvgKey(SvgSensitiveItem):
    '''Base class for key item'''    

    pressShort = pyqtSignal()
    '''Signal for a short press of the key'''
    
    pressLong = pyqtSignal()
    '''Signal for a long press of the key'''
    
    pressRepeat =  pyqtSignal(int)
    '''signal for repetitions of key presses'''
    
    
    
    @classmethod
    def factory(cls, svg, owner, renderer, parent=None):
        '''class method for constructing an object according to its name
        
        :param str oid: The item ID
        :param QSvgPanel owner: The item's owner panel
        :param QWidget parent: optionally the parent widget
        
        :return: The new descendant object of the "SvgItem" class.
        :rtype: SvgItem
        '''
        k = svg.get('id').split('_')
        name = k[1]
        try:
            ext = k[2:]
        except KeyError:
            ext = None
               
        if name == 'dassym':
            return DassymKey(svg, owner, renderer, parent=parent)
        elif name == 'inc':
            return IncrementKey(svg, owner, renderer, parent=parent)
        elif name == 'dec':
            return DecrementKey(svg, owner, renderer, parent=parent)
        elif name == 'parameters':
            return ParametersKey(svg, owner, renderer, parent=parent)
        elif name == 'reverse':
            return ReverseKey(svg, owner, renderer, parent=parent)
        elif name == 'light':
            return LightKey(svg, owner, renderer, parent=parent)
        elif name == 'memory':
            return MemoryKey(svg, owner, num=int(ext[0]),  renderer=renderer, parent=parent)
        elif name == 'workspace':
            if len(ext) > 1 and ext[1].lower() == 'toggle':
                return WorkspaceToggleKey(svg, owner, num=int(ext[0]),  renderer=renderer, parent=parent)
            else:
                return WorkspaceKey(svg, owner, num=int(ext[0]),  renderer=renderer, parent=parent)
        elif name == 'gear':
            return GearKey(svg, owner, numerator=int(ext[0]), denominator=int(ext[1]), renderer=renderer,  parent=parent)
        else:
            return SvgKey(svg, owner, renderer=renderer, parent=parent)
        
    
    def __init__(self, svg, owner,  renderer, parent=None):
        '''Constructor'''
        SvgSensitiveItem.__init__(self, svg, owner, renderer, parent=parent)

    def mousePressEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        self._repeat_count = 0
        self.owner._key_timer.timeout.connect(self.onTimer)
        self.owner._key_timer.start()
        self.owner.log.debug(self.__class__.__name__+":" + self.id + ' - mousePressEvent()')
        super().mousePressEvent(event)
        event.accept()


    def mouseReleaseEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        self.owner.log.debug(self.__class__.__name__+":" + self.id + ' - mouseReleaseEvent()')
        self.owner._key_timer.timeout.disconnect(self.onTimer)
        self.owner._key_timer.stop()
        if self._repeat_count == 0:
            self.pressShort.emit()
        super().mouseReleaseEvent(event)
        
    def onTimer(self):
        self.owner.log.debug(self.__class__.__name__+":" + self.id + ' - onTimer()')
        self._repeat_count += 1
        if self._repeat_count >= KEY_FIRST_REPEAT_COUNT:
            self.pressRepeat.emit(self._repeat_count)
        if self._repeat_count == KEY_LONG_COUNT:
            self.pressLong.emit()

        
class DassymKey(SvgKey):
    '''Class for *Dassym* key'''
    pass
    
class ReverseKey(SvgKey):
    '''Class for *reverse* key'''
    pass

class LightKey(SvgKey):
    '''Class for *light* key'''
    pass

class MemoryKey(SvgKey):
    '''Class for *memory* key'''
    
    def __init__(self, svg, owner, num, renderer, parent=None):
        SvgKey.__init__(self, svg, owner, renderer, parent=parent)
        self.num = num
        

class WorkspaceKey(SvgKey):
    '''Class for *Workspace* key'''
    
    def __init__(self, svg, owner, num, renderer, parent=None):
        SvgKey.__init__(self, svg, owner, renderer, parent=parent)
        self.num = num

class WorkspaceToggleKey(WorkspaceKey):
    '''Class for *Workspace* key'''
    pass
    

class GearKey(SvgKey):
    '''Class for *gear* key'''
    
    def __init__(self, svg, owner, numerator, denominator, renderer, parent=None):
        SvgKey.__init__(self, svg, owner, renderer, parent=parent)
        self.numerator = numerator
        self.denominator = denominator


class IncrementKey(SvgKey):
    '''Class for *increment* key'''
    pass

class DecrementKey(SvgKey):
    '''Class for *decrement* key'''
    pass

class ParametersKey(SvgKey):
    '''Class for *parameters* key'''
    pass


class SvgLed(SvgItem):
    @classmethod
    def factory(cls, svg, owner, renderer, parent=None):
        '''class method for constructing an Led object according to its name
        
        :param str oid: The item ID
        :param QSvgPanel owner: The item's owner panel
        :param QWidget parent: optionally the parent widget
        
        :return: The new descendant object of the :class:`Display` class.
        '''
    
        k = svg.get('id').split('_')
        name = k[1]
        try:
            ext = k[2:]
        except KeyError:
            ext = None

        if name == 'reverse':
            return ReverseLed(svg, owner, renderer, parent=parent)
        elif name == 'light':
            if ext[0] == 'blue':
                return LightBlueLed(svg, owner, renderer, parent=parent)
            else:
                return LightLed(svg, owner, renderer, parent=parent)
        elif name == 'mem':
                return MemoryLed(svg, owner, num=int(ext[0]), renderer=renderer, parent=parent)
            
    def __init__(self, svg, owner, renderer, parent=None):
        SvgItem.__init__(self, svg, owner, renderer, parent=parent)
        self.on = False
        try:
            self._fillOn = self.style['fill']
        except:
            self._fillOn = '#ccff00'
            
        self._fillOff = '#808080'  
            
    def _update(self):
        if self.on:
            self.style['fill'] = self._fillOn
        else: 
            self.style['fill'] = self._fillOff
        
        self.svg.set('style', ';'.join( k+':'+v for k,v in self.style.items() ) ) 
        #self.owner.log.debug('style='+self.svg.get('style'))
        super()._update()
        
        
class ReverseLed(SvgLed):

    def _update(self):
        self.on = self.owner.board.isMotorReverse()
        super()._update()

class LightLed(SvgLed):
    def _update(self):
        self.on = self.owner.board.isLightEnabled()
        SvgLed._update(self)

class LightBlueLed(LightLed):
    
    def initialize(self):
        self.setVisible(self.owner.board.hasBlueLight())
        
    
    def _update(self):
        self.on = self.owner.board.isLightAlternate()
        SvgLed._update(self)

class MemoryLed(SvgLed):
    
    def __init__(self, svg, owner, num, renderer, parent=None):
        SvgLed.__init__(self, svg, owner, renderer, parent=parent)
        self.num = num
        

class Display(SvgItem):
    
    @classmethod
    def factory(cls, svg, owner, renderer, parent=None):
        '''class method for constructing an Display object according to its name
        
        :param str oid: The item ID
        :param QSvgPanel owner: The item's owner panel
        :param QWidget parent: optionally the parent widget
        
        :return: The new descendant object of the :class:`Display` class.
        '''
    
        k = svg.get('id').split('_')
        kind = k[1].lower()

        try:
            ndigits = int(k[2])
        except KeyError:
            ndigits = 3
        
        if kind == '7seg':
            return Display7seg(svg, owner, ndigits=ndigits, renderer=renderer, parent=parent)
        else:
            return Display(svg, owner, ndigits=ndigits, renderer=renderer, parent=parent)
                
    def __init__(self, svg, owner, ndigits=3, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.ndigits = ndigits
        self._text = ''
        
        
    def paint(self, painter, option, widget):
        '''
        
        :param QPainter painter:
        :param QStyleOptionGraphicsItem option:
        :param QWidget widget:
        ''' 
        
        ret = super().paint(painter, option, widget)
        
        rect = QRectF(self.boundingRect())
        #rect = rect.marginsRemoved(QMarginsF(1,2,2,1))
        
        #txt = QStaticText() 
        #txt.setText( "{0:5d}".format(self._speed) )
        #txt.setTextWidth(rect.width())
        txt = self._text
        painter.setPen(Qt.white);
        painter.setBrush(Qt.white);
        
        fm = painter.fontMetrics()
        f = painter.font()
        rectTxt = fm.boundingRect('00000')
        scale = rect.width() / rectTxt.width()
        f.setPointSizeF(f.pointSizeF() * scale)
        painter.setFont(f)
        painter.drawText(rect, Qt.AlignCenter, txt)
        
        return ret
        
        
    def setText(self, text):
        self._text = text
        

class Display7seg(Display):
    pass

class Gauge(SvgItem):
    
    @classmethod
    def factory(cls, svg, owner, renderer, parent=None):
        '''class method for constructing an Gauge object according to its name
        
        :param str oid: The item ID
        :param QSvgPanel owner: The item's owner panel
        :param QWidget parent: optionally the parent widget
        
        :return: The new descendant object of the :class:`Gauge` class.
        '''
    
        k = svg.get('id').split('_')
        kind = k[1].lower()

        if kind == 'torque':
            return GaugeTorque(svg, owner, renderer=renderer, parent=parent)
        elif kind == 'speed':
            return GaugeSpeed(svg, owner, renderer=renderer, parent=parent)
        elif kind == 'speedref':
            return GaugeSpeedReference(svg, owner, renderer=renderer, parent=parent)
        elif kind == 'pressure':
            return GaugePressure(svg, owner, renderer=renderer, parent=parent)
        elif kind == 'analoginput':
            return GaugeAnalogInput(svg, owner, int(k[2]), renderer=renderer, parent=parent)
        elif kind == 'temp':
            return GaugeTemperature(svg, owner, int(k[2]), renderer=renderer, parent=parent)
        elif kind == 'psvoltage':
            return GaugePSVoltage(svg, owner, renderer=renderer, parent=parent)
        else:
            raise KeyError('Unknow gauge kind ({0!s})'.format(kind))
                
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.caption = 'Gauge' 
        self.minValue = 0.0
        self.maxValue = 1.0
        self._value = self.minValue
        
        
    def paint(self, painter, option, widget):
        '''
        
        :param QPainter painter:
        :param QStyleOptionGraphicsItem option:
        :param QWidget widget:
        ''' 
        
        ret = super().paint(painter, option, widget)
        
        rect = QRectF(self.boundingRect())
        scale = (self.value-self.minValue) / (self.maxValue-self.minValue)
        if rect.width()>rect.height():
            rect.setWidth(rect.width() * scale)
        else: 
            h = rect.height() * scale
            rect.setY(rect.height() -h)
            rect.setHeight(h)

        painter.setPen(Qt.NoPen);
        painter.setBrush(Qt.blue);


        painter.drawRect(rect)
        
        return ret
    
    def setValue(self, value):
        if value > self.maxValue:
            self._value = self.maxValue
        elif value < self.minValue:
            self._value = self.minValue
        else:
            self._value = value
        
    
    @property
    def value(self):
        return self._value
    @value.setter
    def value(self, value):
        self.setValue(value)

class BaseGaugeAnalogInput(Gauge):
    '''Gauge to display actual torque (load)'''
    
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
    
    def initialize(self):
        Gauge.initialize(self)
        self.minValue = 0
        self.maxValue = 10000
        self.value = self.minValue

    
class GaugeAnalogInput(BaseGaugeAnalogInput):
    '''Gauge to display actual torque (load)'''
    
    def __init__(self, svg, owner, index, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.index = index
        self.caption = 'Analog #'+str(index)
        
    def _update(self):
        self.value = self.owner.board.getAnalogInput(self.index)
        self.setToolTip("{0:s}: {1:.0f}".format(self.caption, self.value))        
        #self.owner.log.debug('GaugeAnalogInput.value = '+str(self.owner.board.getAnalogInput(self.index)))
        Gauge._update(self)
    

class GaugeTemperature(GaugeAnalogInput):
    '''Gauge to display actual torque (load)'''
    
    def __init__(self, svg, owner, index, renderer=None, parent=None):
        super().__init__(svg, owner, index, renderer, parent)
        self.caption = 'Temp. on An#'+str(index)
        
    def _update(self):
        self.value = self.owner.board.getAnalogInput(self.index)
        self.setToolTip("{0:s}: {1:.1f}°C".format(self.caption, self.value/100))        
        #self.owner.log.debug('GaugeAnalogInput.value = '+str(self.owner.board.getAnalogInput(self.index)))
        Gauge._update(self)

        
class GaugeSpeedReference(BaseGaugeAnalogInput):
    '''Gauge to display actual speed reference'''
    
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.caption = 'Speed ref'
        
    def _update(self):
        try:
            self.value = 10000 * self.owner.board.getSpeedReference() / self.owner.board.motorSpeed() 
        except ZeroDivisionError:
            self.value = 0 
        self.setToolTip("{0:s}: {1:.1f}%".format(self.caption, self.value/100))
        Gauge._update(self)
    

class GaugePressure(BaseGaugeAnalogInput):
    '''Gauge to display actual pressure on sensor'''
    
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.caption = 'Pressure ref'
        
    def _update(self):
        self.value = self.owner.board.getPressure()
        self.setToolTip("{0:s}: {1:.1f}%".format(self.caption, self.value/100))
        Gauge._update(self)
        


class GaugeTorque(Gauge):
    '''Gauge to display actual torque (load)'''
    
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.caption = 'Torque'
        
    def _update(self):
        self.value = self.owner.board.motorRealCurrent()
        self.setToolTip("{0:s}: {1:.0f}mA".format(self.caption, self.value))
        Gauge._update(self)
        
    
    def initialize(self):
        Gauge.initialize(self)
        self.maxValue = self.owner.board.torque_range.upper
        
        
class GaugeSpeed(Gauge):
    '''Gauge to display proportion between actual and set point speed'''
    
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.caption = 'Speed'
        
    def _update(self):
        if self.owner.board.motorSpeed() != 0:
            self.value = self.owner.board.motorRealSpeed() / self.owner.board.motorSpeed()
            self.setToolTip("{0:s}: {1:.1f}% {2:d}".format(self.caption, self.value*100, self.owner.board.motorSpeed()))
        else:
            self.setToolTip("{0:s}: N/A".format(self.caption,))
        Gauge._update(self)
    
class GaugePSVoltage(Gauge):
    '''Gauge to display the power supply voltage'''
    
    def __init__(self, svg, owner, renderer=None, parent=None):
        super().__init__(svg, owner, renderer, parent)
        self.caption = 'PS voltage'
        
    def _update(self):
        self.value = self.owner.board.getPowerSupply()
        self.setToolTip("{0:s}: {1:.1f}V".format(self.caption, self.value))
        Gauge._update(self)
        
    def initialize(self):
        Gauge.initialize(self)
        self.maxValue = self.owner.board.powersupply_range.upper
    

class QSvgPanel(QGraphicsView):
    '''Class for panel with SVG layout
    
    :param QWidget parent: The parent widget.
    :parem list flags: Arguments are passed to the QWidget constructor.    
    '''


    def __init__(self, parent=None):
        '''Constructor'''
        QGraphicsView.__init__(self, parent=parent)
        self.log = logging.getLogger(self.__class__.__name__) 
        self._scene = QGraphicsScene(self)
        self._layout_renderer = QSvgRenderer()
        self._sensitive_renderer = QSvgRenderer()
        self._display_renderer = QSvgRenderer()
        self._key_timer = QTimer()
        self._key_timer.setInterval(KEY_REPEAT_PERIOD)
        self.keys = []
        self.leds = []
        self.gauges = []
        self.setScene(self._scene)
        self._board = None
        self._display = None
        
    def setSvg(self, fname):
        '''Sets the SVG layout and initialize sensitive regions and keys
        
        :param str fname: The SVG file name
        '''

        namespaces={
            #'dc':"http://purl.org/dc/elements/1.1/",
            #'cc':"http://creativecommons.org/ns#",
            #'rdf':"http://www.w3.org/1999/02/22-rdf-syntax-ns#",
            'svg':"http://www.w3.org/2000/svg",
            #'sodipodi':"http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd",
            'inkscape':"http://www.inkscape.org/namespaces/inkscape",    
            }

        xSvgTree = ET.parse(fname)
        xSvgRoot = xSvgTree.getroot()
        
        xLayout = xSvgRoot.xpath("//svg:g[@id='layout']", namespaces=namespaces)[0]
        
         
        self._layout_renderer.load(QByteArray(b'<svg:svg>'+ET.tostring(xLayout)+b'</svg:svg>'))
        self._layout_renderer.setAspectRatioMode(Qt.KeepAspectRatio)
        self._layout_renderer.setViewBox(self.rect())
        
        layout = SvgLayout(xLayout, self, self._layout_renderer)
        self._scene.addItem(layout)
        
        xSensitive = xSvgRoot.xpath("//svg:g[@id='sensitive']", namespaces=namespaces)[0]
        self._sensitive_renderer.load(QByteArray(b'<svg:svg>'+ET.tostring(xSensitive)+b'</svg:svg>'))
        self._sensitive_renderer.setAspectRatioMode(Qt.KeepAspectRatio)
        self._sensitive_renderer.setViewBox(self.rect())

        xKeys = xSensitive.xpath(".//*[starts-with(@id,'key_')]", namespaces=namespaces) 
        
        for xKey in xKeys:
            key =  SvgKey.factory(xKey, self, self._sensitive_renderer)
            # key.pressRepeat.connect(partial(self.onKeyPressRepeat, key))
            # key.pressShort.connect(partial(self.onKeyPressShort, key))
            # key.pressLong.connect(partial(self.onKeyPressLong,key))
            self._scene.addItem(key)
            self.keys.append(key)
            
        self.xDisplay = xSvgRoot.xpath("//svg:g[@id='display']", namespaces=namespaces)[0]
        self._display_renderer.load(QByteArray(b'<svg:svg>'+ET.tostring(self.xDisplay)+b'</svg:svg>'))
        self._display_renderer.setAspectRatioMode(Qt.KeepAspectRatio)
        self._display_renderer.setViewBox(self.rect())
        try:
            xDisplayDisplay = self.xDisplay.xpath(".//*[starts-with(@id,'display_')]", namespaces=namespaces)[0]
            self._display = Display.factory(xDisplayDisplay, self, self._display_renderer)
            self._scene.addItem(self._display)
        except IndexError:
            self._display = None
            
        for xLed in self.xDisplay.xpath(".//*[starts-with(@id,'led_')]", namespaces=namespaces):
            led =  SvgLed.factory(xLed, self, self._display_renderer)
            self._scene.addItem(led)
            self.leds.append(led)
            
        for xGauge in self.xDisplay.xpath(".//*[starts-with(@id,'gauge_')]", namespaces=namespaces):
            gauge =  Gauge.factory(xGauge, self, self._display_renderer)
            self._scene.addItem(gauge)
            self.gauges.append(gauge)
            
    def setBoard(self, board):
        self._board = board
        self.initialize()
            
    def initialize(self):
        if self._board is None: return
        for led in self.leds:
            led.initialize()
        for gauge in self.gauges:
            gauge.initialize()        
        
        
    def refresh(self):
        if self._board is not None:
            self._display._update()
            self._display_renderer.load(QByteArray(b'<svg:svg>'+ET.tostring(self.xDisplay)+b'</svg:svg>'))
            for led in self.leds:
                led._update()
            for gauge in self.gauges:
                gauge._update()
            
        
    def update(self, *args, **kwargs):
        self.refresh()        
        ret = QGraphicsView.update(self, *args, **kwargs)
        return ret
          
            
    #===========================================================================
    # def onKeyPressShort(self, key):
    #     self.log.debug('onKeyPressShort('+str(key.id)+")")
    #     
    # def onKeyPressRepeat(self, key, count):
    #     self.log.debug( 'onKeyPressRepeat('+str(key.id)+", "+str(count)+")" )
    #     
    # def onKeyPressLong(self, key):
    #     self.log.debug( 'onKeyPressLong('+str(key.id)+")" )
    #===========================================================================

        
    def resizeEvent(self, *args, **kwargs):
        '''Rewrite the resize event handler'''
        self.fitInView(self._scene.sceneRect(), Qt.KeepAspectRatio)
        return QGraphicsView.resizeEvent(self, *args, **kwargs)

    # def mousePressEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        # #self.log.debug('SvgViewer - mousePressEvent() - X:'+str(event.x())+";Y:"+str(event.y()) )
        # super().mousePressEvent(event)
        
        
    def mouseReleaseEvent(self, event: 'QtWidgets.QGraphicsSceneMouseEvent'):
        '''Rewrite the mouse release event handler'''
        self.log.debug('SvgViewer - mouseReleaseEvent()')
        super().mouseReleaseEvent(event)
        
        
    def setDisplayText(self, txt):
        if self._display:
            self._display.setText(txt)
            
        
    @property
    def scene(self):
        return self._scene
    @property
    def renderer(self):
        self._renderer
        
    @property
    def board(self):
        return self._board
        
        
        


'''Definition of base class for GUI application and base class for main window.

:author: F. Voillat
:date: 2021-0-24 Creation
:copyright: Dassym 2021
'''

from PyQt5.Qt import QApplication, QMessageBox, QAction, QSettings, QObject, QMainWindow,\
    Qt, QTimer, QTime, QEventLoop, QGuiApplication, QIcon, QSize
import sys
from functools import partial
from os.path import os

from app.base import BaseApp
import app
from app.gui.dlg import AboutDialog
from dapi2.dcom.base import DComException
from dboard.workspace import BaseWorkspace
from app.ressource import getRessourcePath
from app.gui.dlg.selectconnectiondialog import SelectConnectionDialog

from .res import res_img #@UnusedImport
from app.gui.splashscreen import AppSplashScreen
from dapi2.derror import DApiComError

 
TIMEOUT_MS = 250
'''GUI timer delay [ms]'''

REFRESH_COUNT = 2
'''Count between 2 refresh'''


STANDARD_LAYOUTS = {
    'vantage': os.path.join(app.IMG_DIR, 'layout-vantage.svg'),
    'vantage-dark': os.path.join(app.IMG_DIR, 'layout-vantage-dark.svg'),
    'lipo': os.path.join(app.IMG_DIR, 'layout-lipo.svg'),
    }


#===============================================================================
# class BaseGuiObject(QObject):
#     '''Base class for GUI objects.'''
#         
#     def __init__(self, owner=None, parent=None, widget=None):
#         '''Initialization
#         
#         :param owner: The owner of this GUI object (BaseGuiObject).
#         :param parent: The parent widget for displaying this GUI object (QWidget).    
#         :param widget: The widget if already created (QWidget).
#         '''
#         assert isinstance(owner, BaseGuiObject) or owner == None
#         assert isinstance(parent, QWidget) or parent == None
#         assert isinstance(widget, QWidget) or widget == None
#         self._owner = owner
#         self._parent = parent
#         self._widget = widget
#         self._disable_events_cnt = 0
#         super().__init__()
#         
#     def getWidget(self, name, qclass = QObject): 
#         ret = self.findChild(qclass, name, options=Qt.FindChildrenRecursively)
#         if ret == None: 
#             raise Exception('Widget `{0!s}` ({1!s}) not found!'.format(name, qclass.__name__))
#         return ret
#             
#     def disableEvents(self):
#         self._disable_events_cnt += 1
#         return self._disable_events_cnt
#     
#     def enableEvents(self):
#         if self._disable_events_cnt > 0:
#             self._disable_events_cnt -= 1
#         return self._disable_events_cnt
# 
#     def setSensitive(self, value):
#         self._widget.set_sensitive(value)
#         
#         
#     def _saveGeometries(self, settings):
#         pass
# 
#     def _restoreGeometries(self, settings):
#         pass
#     
#     
#     def saveUserConfig(self, settings):
#         '''Enregistre la configuration utilisateur.'''
#         settings.beginGroup(self.__class__.__name__)
#         try:
#             settings.setValue("Geometry", self.saveGeometry())
#             try:
#                 settings.setValue("State", self.saveState())
#             except AttributeError: pass
#             self._saveGeometries(settings)
#         except Exception as e:
#             self.log.error('save_user_config:'+str(e))
#         settings.endGroup()
#         
#     def restoreUserConfig(self, settings):
#         '''Restaure la configuration utilisateur.'''
#         settings.beginGroup(self.__class__.__name__)
#         try: 
#             self.restoreGeometry(settings.value("Geometry"))
#             try:
#                 self.restoreState(settings.value("State"))
#             except AttributeError: pass
#             self._restoreGeometries(settings)
#         except Exception as e:
#             self.log.error('restore_user_config:'+str(e))
#         settings.endGroup()
#         
#     @property
#     def owner(self):
#         return self._owner
#     @property
#     def parent(self):
#         return self._parent
#     
#     @property
#     def widget(self):
#         return self._widget 
#     @property
#     def events_enabled(self):
#         return self._disable_events_cnt == 0  
#     @property
#     def window(self):
#         return self._owner.window        
#     @property
#     def app(self):
#         return self._owner.app      
#===============================================================================
    

    
def UserActionHandler(function):
    
    def wrapper(self, *args, **kwargs):
        if not self.eventsEnabled: return
        self.log.debug(function.__name__+str(args))
        self.disableEvents()            
        try:
            return function(self, *args, **kwargs)
        except DApiComError as e:
            self.handleDApiComError(e)
        except Exception as e:
            self.handleError(e)
        finally:
            self.enableEvents()    
            return None
    return wrapper

def UserBoardActionHandler(function):
    
    def wrapper(self, *args, **kwargs):
        ret = None
        if not self.eventsEnabled: return
        self.log.debug(function.__name__+str(args))
        self.disableEvents()            
        try:
            ret = function(self, *args, **kwargs)
        except DApiComError as e:
            self.handleDApiComError(e)
        except Exception as e:
            self.handleError(e)
        finally:
            self.enableEvents()    

        self.update()
        return ret 

    return wrapper



class BaseMainWindow(QMainWindow):
    '''Base class for application's main window.
    
    :param BaseGuiApp app: The application that owns the window
    '''
    
    def __init__(self, app, *args, **kwargs):
        '''Constructor'''
        self._app = app
        self._disable_events_cnt = 0
        self._dialogs = {}
        self._actions = {}
        QMainWindow.__init__(self, *args, **kwargs)
        
    def _initWin(self):
        self._addDialog('about', AboutDialog(self))
        self.actionAbout.triggered.connect(self.onAbout)
        self.actionAutoRefresh.setChecked(self._app.autorefresh)
        self.actionAutoRefresh.toggled.connect(self.onAutorefreshToggled)
        self.actionRefresh.triggered.connect(self.onRefreshTriggered)
        
        
    def _cleanupWin(self):
        
        for dlg in list(self._dialogs.keys()):
            try:
                self.log.debug('cleanup dialog '+dlg)
                self._dialogs[dlg].close()
                del self._dialogs[dlg]
            except Exception as e:
                self.log.error('An error occurred while cleaning up the {} dialog!\n{}'.format(dlg, str(e)))
                
        
    def _addAction(self, parent, name, label, data, icon, handler, tooltips=None, menu=None, bar=None):
        '''Adds an action on menu and toolbar.
        
        :param QWidget parent: The parent of new action.
        :param str name: The action name
        :param str label. The action label
        :param data: Data to pass to action
        :param QIcon icon: The action icon
        :param function handler: The action handler  
        :param str tooltips: The action tooltips text
        :param QMenu menu: the menu to which the action must be added
        :param QToolBar bar: the toolbar to which the action must be added
        '''
        
        #self.log.debug('AddAction({0:s})'.format(name))
        
        if icon is not None:
            action = QAction(icon, label, parent, triggered=partial(handler, data) )
        else:
            action = QAction(label, parent, triggered=partial(handler, data) )
        action.setObjectName(name)
        action.setToolTip(tooltips)
        action.setEnabled(True)
        if menu is not None:
            menu.addAction(action)
        if bar is not None:
            bar.addAction(action)        
        #self.log.debug('Add action '+name)
        self._actions[name] = action
        return action  
    
    def _addDialog(self, name, dialog):
        assert not name in self._dialogs, 'The `{}` dialog already exists!'.format(name)
        self.log.debug('_addDialog('+name+','+str(dialog)+')') 
        self._dialogs[name] = dialog      
    
    def initialize(self):
        for qaction in self.findChildren(QAction):
            if qaction.objectName():  
                self._actions[qaction.objectName()] = qaction
        
        for ws in self.app.board.workspaces:
            self._addAction(self, name='actionWorkspace{0:d}'.format(ws.par), label=ws.name, data=ws, icon=None, handler=self.onWorkspaceTriggered, tooltips=None, menu=self.menuWorkspace)
            
        self.update()
        
    def finalize(self):
        self._cleanupWin()
        
    def closeEvent(self, event):
        self.log.debug('closeEvent '+str(event))
        for dlg in list(self._dialogs.keys()):
            self._dialogs[dlg].close()
        event.accept()
        return QMainWindow.closeEvent(self, event)
        
    def saveGeometries(self, settings):
        settings.beginGroup("MainWindow")
        settings.setValue("geometry", self.saveGeometry())
        settings.setValue("state", self.saveState())
        settings.endGroup()
    
        
    def restoreGeometries(self, settings):
        settings.beginGroup("MainWindow")
        self.restoreGeometry(settings.value("geometry"))
        self.restoreState(settings.value("state"))
        settings.endGroup()        
    
    def disableEvents(self):
        '''Disable event processing'''
        self._disable_events_cnt += 1
        return self._disable_events_cnt
    
    def enableEvents(self):
        '''Enable event processing.
        
        For the processing of events to be active, the number of nested calls for deactivation and activation must be equal.
        '''
        if self._disable_events_cnt > 0:
            self._disable_events_cnt -= 1
        return self._disable_events_cnt
    
    
    def getWidget(self, name, qclass = QObject): 
        '''Find a widget based on its name
        
        :param str name: The widget name to find.
        :param class qclass: The widget call to find. (default QObject => any)    
        ''' 
        ret = self.findChild(qclass, name, options=Qt.FindChildrenRecursively)
        if ret == None: 
            raise Exception('Widget `{0!s}` ({1!s}) not found!'.format(name, qclass.__name__))
        return ret
    
    def handleError(self, exception):
        '''Error handler'''
        return self._app.handleError(exception)
    
    def handleDApiComError(self, exception):
        '''DAPI communication Error handler'''
        self.log.error(str(exception))
        self.app.displayError(exception)
        
    
    def refreshActionsSensitivity(self, context={}):
        '''Updates the sensitivity of actions
        
        :param dict context: A dictionary that can contain contextual information.
        '''
        available_actions =  self.app.getAvailableActions(context)
        for aname in available_actions:
            o = self.getWidget(aname)
            o.setEnabled(True)
        for aname in self.actions - available_actions:
            try:
                self.getWidget(aname).setEnabled(False)
            except Exception as e:
                self.log.error('refreshActionsSensitivity -> '+str(self.actions - available_actions))
                raise e   
            
            
    
    def refresh(self):
        '''Refresh the windows according board setpoints'''
        self.disableEvents()
        try:
            if not self.board.isOnStandby():
                self.board.refreshSetpoints()
            for action in self._actions.values():
                if isinstance(action.data, BaseWorkspace):
                    action.setChecked( action.data.active )
                    
            self.refreshActionsSensitivity()
        finally:
            self.enableEvents()
        
    
    @UserActionHandler
    def onAbout(self, checked=None):
        '''Handler for "about" action.'''
        self.dialogs['about'].exec()
    
    @UserActionHandler
    def onRefreshTriggered(self, checked=None):
        self.app.refresh()
        
    
    @UserActionHandler
    def onAutorefreshToggled(self, checked=None):
        self.app.autorefresh = checked
    
    @UserBoardActionHandler
    def onWorkspaceTriggered(self, workspace=None, checked=None):
        self.app.board.setWorkspace(workspace)
    
        
    @property
    def app(self):
        '''The application that owns the window.'''
        return self._app
    @property
    def log(self):
        '''shortcuts to the logger of the application'''
        return self._app.log
    @property
    def board(self):
        '''shortcuts to the board (:class:`dboard.BaseDBoard`) of the application'''
        return self._app.board
    @property
    def dapi(self):
        '''shortcuts to the *dapi* (:class:`dapi2.dapi2.DApi2`) of the application'''
        return self._app.dapi
    @property
    def eventsEnabled(self):
        '''The events processing activation status'''
        return self._disable_events_cnt == 0          
    @property
    def actions(self):
        '''The list of names of actions'''
        return self._actions.keys()
    
    @property
    def dialogs(self):
        '''The dialog windows dictionary {name:widget}'''
        return self._dialogs
    

class BaseGuiApp(BaseApp, QApplication):
    '''Base class for GUI applications
    
    :param Namespace config: Application configuration
    '''
    
    NAME = 'dcp-base-gui'
    '''Application name'''
    
    @classmethod
    def cast(cls, app):
        assert isinstance(app, BaseApp)
        app.__class__ = cls
        return app

    def __init__(self, app_dir):
        '''Constructor'''
        QApplication.__init__(self, [])
        self.splash = None 
        BaseApp.__init__(self, app_dir)
        self.autorefresh = True
        self._refresh_count = REFRESH_COUNT
        self.setApplicationVersion(app.__version__)
        self.setApplicationName(self.NAME)
        self.setOrganizationName(app.DASSYM)
        self.setOrganizationDomain(app.DASSYM_DOMAIN)
        self.setQuitOnLastWindowClosed(True) 
        self._window = None
        self.openSplashScreen()
        self._timer = QTimer()
        
        
        
    def _initWin(self):
        assert False, "Abstract method!"
        
    def connectionCallback(self, options):  
        dialog = SelectConnectionDialog(options)
        r = dialog.exec()
        if r == True:
            ret = dialog.getSelectedOption()
            del dialog
            return int(ret)
        else:
            sys.exit(0)
    
        
    def _refresh(self):
        if self._board is not None and self.isConnected():
            if not self._dcom.isOk(): return 
            self._board.refreshState()
            #self._board.refreshSetpoints()
            #TODO: self.update_status_misc()
        # else:
            # self._label_status_miscellaneous.setText('~')
            # self._label_status_connection.setText('~')
            
    def prepareSpecificCfg(self, parser):
        parser.add_argument("--layout",  help=BaseApp.tr("The GUI layout name or SVG file. (default: vantage)"), dest="layout", default="vantage")
    
            
    def getAvailableActions(self, context={}):
        '''Returns a set with available actions according actual situation.
        
        :return: available actions
        :rtype: Set of str 
        '''
        ret = set(['actionQuit','actionHelp','actionAbout', 'actionAutoRefresh'])
        if self._board is not None: 
            if self.isConnected():
                ret |= set(('actionSysDisconnect', 'actionRefresh'))
                aws = self._board.getWorkspace()
                for ws in self._board.workspaces:
                    if ws != aws:
                        ret.add('actionWorkspace{0:d}'.format(ws.par)) 
                if aws.isFunctional(): 
                    ret |= set(['actionMotorCW','actionMotorCCW', 'actionLightEnabled',])
                    
                    if self._board.hasBlueLight():
                        ret.add('actionLightBlue')
                    if self._board.isMotorStarted():
                        ret.add('actionMotorStop')
                    else:                        
                        ret.add('actionMotorStart')
            else:
                ret.add('actionSysConnect')
        return ret     
    
    def openSplashScreen(self):
        '''Ouvre le splash screen''' 
        self.log.debug('openSplashScreen')
        #splash_img = QPixmap(':img/splashscreen.png')
        rect = QGuiApplication.primaryScreen().geometry()
        splash_img = QIcon(':img/splashscreen.svg').pixmap(QSize(400,200))
        self.splash = AppSplashScreen(self, splash_img, Qt.WindowStaysOnTopHint)
        self.splash.setWindowFlags(Qt.WindowStaysOnTopHint | Qt.FramelessWindowHint)
        self.splash.setEnabled(True)
        self.splash.show()
        self.splash.showMessage(self.applicationName())
        self.splash.show()
        self.splash.update()
        self.processEvents(QEventLoop.ProcessEventsFlag.AllEvents)
        self.log.debug('splashscreen open')
        return self.splash 
        
        
    def closeSplashScreen(self):
        '''Ferme le splash screen'''
        self.log.debug('closeSplashScreen')
        self.processEvents()
        self.splash.finish(self._window)
        self.splash = None      
        
    def say(self, level, text):
        
        super().say(level,text)
        if self.splash:
            self.splash.showMessage(text)
            self.processEvents(QEventLoop.ProcessEventsFlag.AllEvents)
        
        
    def saveUserConfig(self):
        '''Saves the user configuration of windows of application.'''
        self.log.debug('saveUserConfig')
        settings = QSettings()
        self._window.saveGeometries(settings)

    def restoreUserConfig(self):
        '''Restores the user configuration of windows of application.'''
        self.sayHigh(BaseApp.tr('Restoration of visual configuration'))
        settings = QSettings()
        try:
            self._window.restoreGeometries(settings)
        except Exception as e:
            self.log.error(str(e))
            self.sayError(BaseApp.tr('Restoration of visual configuration is aborted!'))


    def refresh(self):
        '''Refresh the display'''
        self._refresh()
        self._window.refresh()
        

    def initialize(self):
        '''Initialization of application'''
        self._initWin()
        BaseApp.initialize(self)
        self.sayHigh('Load board registers...')
        self.board.refreshAll()
        self.sayHigh('Window initialization...')
        self._window.initialize()
        
        
        
    def run(self):
        '''Executes application'''
        self.log.info(BaseApp.tr('Run')+" "+self.NAME+"...")
        
        self.log.debug('Show main window...')
        self._window.show()
        self.restoreUserConfig()
        self.closeSplashScreen()
        self._timer0 = self.startTimer(TIMEOUT_MS)
        self.log.debug('Start Qt main loop...')
        exit_code = self.exec()
        self.log.debug('Exit from Qt main loop...')
        self.saveUserConfig()
        self._window.finalize()
        self.killTimer(self._timer0)
        del self._window
        return self.terminate(exit_code)
                

    def terminate(self, exit_code=0):
        '''Terminates application'''
        try:
            self.log.info(BaseApp.tr('Finished')+"("+str(exit_code)+")")
        except:
            pass
        if exit_code > 0:
            self.exit(exit_code)
            exit(exit_code)         
        
        
    def displayError(self, error):
        '''Display the given error
        
        :param str error: The error message''' 
        self.log.error(error)
        dialog = QMessageBox(
                    QMessageBox.Critical,
                    self.applicationName()+BaseApp.tr('Error:'),
                    BaseApp.tr("An error has occurred!\nDetail:\n")+str(error),
                    QMessageBox.Ok )
        dialog.exec()
        
    def timerEvent(self, event=None):
        '''Periodic processing caused by the GUI "timeout"
        
        @param QTimerEvent event : The timer event.
        '''
        try:
            #self.log.debug('timer0')
            if self.autorefresh and self._refresh_count > 0:
                self._refresh_count -= 1
                if self._refresh_count == 0 :
                    self.refresh()
                    self._refresh_count = REFRESH_COUNT
        except DComException as e:
            self.board.disconnect()
            self.refresh()
        except Exception as e:
            self.handleError(e)
            
            
    def getLayoutFile(self, layout):
        '''Returns the SVG file for a control panel layout
        
        :param str layout: The layout name or resource
        :return: the SVG file path
        :rtype: str
        '''  
        try:
            return STANDARD_LAYOUTS[layout]
        except KeyError:
            return getRessourcePath(layout)
        
        
            
    def sleep(self, seconds):
        '''Put the application to sleep
        
        :param float seconds: sleep duration of the application expressed in seconds
        '''
        wake_up_time = QTime.currentTime().addMSecs(int(seconds*1000))
        while (QTime.currentTime() < wake_up_time):
            self.processEvents(QEventLoop.AllEvents, 100)        
        
        
    @property
    def window(self):
        '''The main window of application'''
        return self._window


        
        


'''Launcher for *dsmcp developer*. 


:author: F. Voillat
:date: Created on 24 mars 2021
'''

import sys
from os.path import os


if __name__ == '__main__':
    if sys.version_info < (3,8):
        raise Exception("DSMCP requires at least Python version 3.8")

    from app.launcher import Launcher
    launcher = Launcher(os.path.dirname(__file__))
    launcher.preConfig()
    if launcher.config.no_gui:
        from app.cli import DevCliApp
        application = launcher.launch(DevCliApp) 
    else:
        from app.gui import DevGuiApp
        application = launcher.launch(DevGuiApp)  
    try: 
        application.initialize()
        application.run()
    except Exception as e:
        application.handleError(e)
        sys.exit(1)


'''

:author: fv
:date: Created on 19 mars 2021
'''
import logging.handlers
import os.path
import re as RE
from serial import Serial
import traceback
import platform
import getpass
import sys
import socket
from PyQt5.QtCore import QCoreApplication, QLocale, QTranslator,\
    QLibraryInfo, QT_VERSION_STR, PYQT_VERSION_STR


import dapi2, dboard
from .tracer import Tracer
from dapi2.dcom.dsocket import DSocket
from .common import IMG_DIR, FIRM_DIR, TEMP_DIR, USR_DATA_DIR, APP_NAME, VERBOSITY, LOG_FORMAT, DEFAULT_LOG_DIR
from ._version import __version__
from .firmware import Firmwares
from .utils import configStr
 
class BaseApp(object):
    '''Base class for *dsmcp* applications
    
    :param str app_dir: The application root directory.
    
    '''
    
    NAME = APP_NAME

    @classmethod
    def tr(cls, text, disambiguation=None, n=-1):
        return QCoreApplication.translate('BaseApp',text, disambiguation, n)

    

    def __init__(self, app_dir):
        '''
        Constructor
        '''
        self.log = logging.getLogger(self.__class__.__name__)
        self.config = None
        self.app_dir = app_dir
        self.log_to_file = False
        self._dcom = None
        self._dapi = None
        self._board = None
        self._tracer = Tracer(self) 
        self._firmwares = Firmwares(self)
        
        #self._board = Board92(self, self.registers)


    def _initDCom(self):
        if self.config.host is not None:
            return self._initDSocket()
        else:
            port = self.config.serial
            if not os.path.exists(self.config.serial):
            
                m0 = RE.match('(.*)(\d+)', self.config.serial)
                bport = m0.group(1)
                nport = int(m0.group(2)) 
                for i in range(nport, nport+10):
                    port = '{0:s}{1:d}'.format(bport, i)
                    if os.path.exists(port):
                        break
                    port = None
            
                if port is None:
                    raise Exception(BaseApp.tr('No serial port found between {0:s}{1:d} and {0:s}{2:d} !').format(bport, nport, i) )
            
            
            
            return self._initDSerial(port)
    
    def _initDSerial(self, port):
        serial_port = Serial(port)
        serial_port.baudrate = self.config.baudrate
        serial_port.timeout = 5
        serial_port.stopbits = 1
        if self.config.trace:  
            return dapi2.DSerial(serial_port, dapi2.DApi2Side.MASTER, trace_callback=self._tracer.traceDapi)
        else:
            return dapi2.DSerial(serial_port, dapi2.DApi2Side.MASTER, trace_callback=None)

        
          
    def _initDSocket(self):
        skt = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        host = self.config.host.split(":")
        
        if len(host) == 0:
            raise Exception("No host specified")
        elif len(host) == 1:
            host.append(30443)
        else:
            host[1] = int(host[1])
        
        skt.connect((host[0], host[1]))
        com = DSocket(skt, dapi2.DApi2Side.MASTER, trace_callback=None)
        options = com.get_ext_conn()
        com.select_ext_conn(self.connectionCallback(options))
        return com
    
    
    
    def prepareCfg(self, parser):
        '''Prepare the argument structure of the application to be parsed by the :meth:`processCfg` method.
        
        :param ArgumentParser parser: The parser to initialize.
        '''
    
        parser.add_argument("-l", "--log-level", dest="loglevel", choices=[ 'error', 'warning', 'debug', 'info', 'noset'], help=BaseApp.tr("Log level. (default = info)"), default="info"   )
        parser.add_argument("--log-file", dest="logfile", default='stderr', help=BaseApp.tr("LOG file (default = stderr)") )
        parser.add_argument("--log-max-size", dest="log_maxbytes", help=BaseApp.tr("Maximum LOG file size in bytes (default=100000)."), default=100000)
        parser.add_argument("--log-backup-count", dest="log_backupcount", help=BaseApp.tr("Number of LOG files retained (default=10)."), default=10)
        parser.add_argument("-t", "--trace", dest="trace", action='store_true', help=BaseApp.tr("If present, trace API message exchanges (default = not)"), default=False )
        
        dcom_grp = parser.add_mutually_exclusive_group(required=True)
        dcom_grp.add_argument("-S", "--serial", dest="serial", help=BaseApp.tr("Serial port for direct control."), metavar="SERIAL")
        dcom_grp.add_argument("-H", "--host", dest="host", help=BaseApp.tr("Host name and port for remote control. (host:port)"), metavar="HOST")
        
        parser.add_argument("-C", "--command-mode", dest="command_mode", action='store_true', help=BaseApp.tr("Sets the preferred DAPI2 mode to 'command'. (Default=command mode)"),default=True )
        parser.add_argument("-R", "--register-mode", dest="command_mode", action='store_false', help=BaseApp.tr("Sets the preferred DAPI2 mode to 'register'. (Default=command mode)"))
        parser.add_argument("-D", "--dev-mode", dest="dev_mode", action='store_true', help=BaseApp.tr("Sets DSMCP execution mode 'development'. (Default=normal mode)"), default=False)
            
        parser.add_argument("-b", "--baudrate", dest="baudrate", type=int, help=BaseApp.tr("Baud rate. (default: {0})".format(dapi2.COM_SPEEDS[0]) ), metavar="BAUDRATE", default=dapi2.COM_SPEEDS[0])
        parser.add_argument("--version", action='version',  help=BaseApp.tr("Show software version"), version='%(prog)s version {0:s}'.format(__version__)   )
        parser.add_argument("--firm-paths", dest="firm_paths", help=BaseApp.tr("Search paths for firmware binaries. The different paths are separated by a semicolon (;)"))
        parser.add_argument("--registers",  help=BaseApp.tr("The XML registers definition file"), dest="regfile",  default="data:dapi-registers.xml" )
        
        self.prepareSpecificCfg(parser)
        
    def prepareSpecificCfg(self, parser):
        pass
        
    def processCfg(self, parser, args):   
        '''Parse the command line arguments and 
        
        :param ArgumentParser parser: The parser.
        :param Namespace args: The arguments default values given by the configuration file (.ini)
        '''
        parser.set_defaults(**args)
        self.config = parser.parse_args()
        
        if self.config.loglevel:
            self.config.loglevel = getattr(logging, self.config.loglevel.upper())
    
        if self.config.verbosity is not None:
                try:
                    if isinstance(self.config.verbosity_cnt, int):
                        self.config.verbosity = VERBOSITY(self.config.verbosity)
                    elif self.config.verbosity.isdigit():
                        self.config.verbosity = VERBOSITY(str(self.config.verbosity))
                    else:
                        self.config.verbosity = VERBOSITY[self.config.verbosity.upper()]
                except:
                    self.config.verbosity = VERBOSITY.NONE
        
        if self.config.verbosity_cnt is not None:
            try:
                self.config.verbosity = VERBOSITY(self.config.verbosity_cnt)
            except:
                pass
        
        if 'firm_paths' in self.config and self.config.firm_paths is not None:
            self.config.firm_paths = self.config.firm_paths.split(';') 
        else:
            self.config.firm_paths = []
            
        self.processSpecificCfg()
            
    def processSpecificCfg(self):
        pass   
     
    def initLog(self):
        '''Initialize the application logger'''
        
        if self.config.logfile is None or self.config.logfile == 'stderr':
            log_handler = logging.StreamHandler(sys.stderr)
        elif self.config.logfile == 'stdout':
            log_handler = logging.StreamHandler(sys.stdout)
        else:
            if os.path.dirname(self.config.logfile) != '':
                logfile = os.path.normpath(self.config.logfile)
            else:
                logfile = os.path.normpath(os.path.join(DEFAULT_LOG_DIR, self.config.logfile))
            log_handler = logging.handlers.RotatingFileHandler( filename=logfile, maxBytes=self.config.log_maxbytes, backupCount=self.config.log_backupcount)
            self.log_to_file = True
    
        self.log.setLevel(self.config.loglevel)
        logfmt = logging.Formatter(LOG_FORMAT)
        log_handler.setFormatter(logfmt)   
        self.log.addHandler(log_handler)
            
    def prepareL10n(self, lang):
        '''Prepare the application localization
        
        :param str lang: If defined the language to use, otherwise the local session language.
        ''' 
        
        if lang:
            self.locale = QLocale(lang)
        else:
            self.locale = QLocale.system()
        self._qtTranslator = QTranslator()
        self._qtBaseTranslator = QTranslator()
        self._qtAppTranslator = QTranslator()        
        
        s = QLibraryInfo.location(QLibraryInfo.TranslationsPath)

        if not self._qtAppTranslator.load(self.locale, 'dsm-cp', "_", ":/i18n/", ".qm"):
            self.log.error('DSMCP translation resource load failed for {0!s}!'.format(self.locale.name()))
        else:
            self.installTranslator(self._qtAppTranslator)
            self.log.info(BaseApp.tr('DSMCP Translation resource successfully loaded for {0!s}.').format(self.locale.name()))
 
        if not self._qtTranslator.load( 'qt_help_fr', s):
            self.log.error('Translation load `qt_help_fr.qm` failed for {0!s} in `{1!s}`!'.format(self.locale.name(), s))
        else:
            self.installTranslator(self._qtBaseTranslator)
            self.log.info(BaseApp.tr('Qt translation file successfully loaded for {0!s}.').format(self.locale.name()))
            
        if not self._qtBaseTranslator.load(self.locale, 'qtbase', '_', s):
            self.log.error('Translation load `qtbase_*.qm` failed for {0!s} in `{1!s}`!'.format(self.locale.name(), s))
        else:
            self.installTranslator(self._qtBaseTranslator)
            self.log.info(BaseApp.tr('Qt base translation file successfully loaded for {0!s}.').format(self.locale.name()))

            
            
    def connectionCallback(self, options):
        assert False, "No overwritted func"
        
    def setBoard(self, board):
        '''Sets the board of application.
        
        :param BaseDBoard board: The board of application
        ''' 
        self._board = board
        
    def initialize(self):
        '''Initialize the application
        
        Actions performed:
        
            - Instantiation of the communication object: a descendant of :class:`~dapi2.dcom.base.BaseDCom`.
            - Instantiation of the DAPI2 API (:class:`~dapi2.dapi2.DApi2`)
            - Instantiation of the object representing the electronic card: a descendant of :class:`dboard.base.BaseDBoard`.
        '''
        self.sayMedium(BaseApp.tr("Start {0:s} v{1:s}").format(self.NAME, __version__))
        self.log.debug('Initialize...')
        self.log.debug('Python version: {} ({})'.format( ".".join([str(x) for x in sys.version_info[:3]]) , platform.architecture()[0]))
        self.log.debug('OS: {}'.format(platform.platform()))
        self.log.debug('Qt version:{0!s}'.format(QT_VERSION_STR))
        self.log.debug('PyQt version:{0!s}'.format(PYQT_VERSION_STR))
        self.log.debug('Qt locale:`{0!s}` ; decimal: `{1!s}` : language: {2!s}'.format(self.locale.name(), self.locale.decimalPoint(), self.locale.bcp47Name()))
        self.log.debug('PyDapi2:{0:s}'.format(dapi2.__version__))
        
        self.log.debug('Configuration:\n\t'+configStr(self.config))
        
        self.sayHigh(BaseApp.tr('Connecting...'))
        self._dcom = self._initDCom()
        self._dapi = dapi2.DApi2(self._dcom, dev_mode=self.isInDevMode() )
        self._tracer.dapi = self._dapi
        if not self.config.command_mode:
            dmode = dboard.DBoardPreferedDapiMode.REGISTER
        else:
            dmode = dboard.DBoardPreferedDapiMode.COMMAND
        self._board = dboard.DBoardFactory(self.dapi, dmode, self.sayLow)
        
        #TODO: m = BaseApp.tr("{a:s}@{b.name}:{b.sn:05d} through {s.name!s}@{s.baudrate:d}").format(
                # a=self._board.access.name, b=self._board, s=self._board.com.serial)
        self.sayMedium(BaseApp.tr("Connected."))
        self.board.initialize()
        self._firmwares.discover(self.config.firm_paths, self.board)
        
        
         
        
        
    def run(self):
        '''Run this application''' 
        return 0
        
    def terminate(self, exit_code=0):
        '''Terminates application'''
        try:
            self.sayMedium(BaseApp.tr('Finished')+"("+str(exit_code)+")")
        except:
            pass
        if exit_code > 0:
            exit(exit_code)
            
    def say(self, level, text):
        '''Display text on stdout
        
        :param VERBOSITY level: The level needed to display this text.
        :param str text: Text to display.
        '''
        if self.config.verbosity >= level:
            print( text )
            if self.log_to_file:
                if level >= VERBOSITY.MEDIUM:
                    self.log.debug( text )
                else:
                    self.log.info( text )
        else:
            if level >= VERBOSITY.MEDIUM:
                self.log.debug( text )
            else:
                self.log.info( text )
        
    def sayMinimal(self, text):
        self.say(VERBOSITY.MINIMAL, text)
    def sayLow(self, text):
        self.say(VERBOSITY.LOW, text)
    def sayMedium(self, text):
        self.say(VERBOSITY.MEDIUM, text)
    def sayHigh(self, text):
        self.say(VERBOSITY.HIGH, text)
      
    def sayError(self, text):
        if self.config.verbosity >= VERBOSITY.MINIMAL:
            print( self.tr('ERROR:')+str(text) )
            if self.log_to_file:
                self.log.error(text)
        else:
            self.log.error(text)
            
    def sayWarning(self, text):
        if self.config.verbosity >= VERBOSITY.MINIMAL:
            print( self.tr('WARNING:')+str(text) )
            if self.log_to_file:
                self.log.warning(text)
        else:
            self.log.warning(text)        
            
         
            
    def displayError(self, error):
        '''Display the given error
        
        :param str error: The error message''' 
        self.sayError(error)
        
        
    def handleError(self, exception):
        '''Handling an error or exception
        :param Exception e: the exception to be processed
        '''
        txt = str(exception)
        self.displayError(txt)
        txt += '\n'+ str(traceback.format_exc())
        txt += '\n{0!s} : {1!s}'.format(self.applicationName(), self.applicationVersion())
        txt += '\nPython : {0!s}'.format( ".".join([str(x) for x in sys.version_info[:3]]) )
        txt += '\nStation : {0!s} / {1!s}'.format(platform.node(), platform.platform())
        txt += '\nUser : {0!s}'.format(getpass.getuser())
        self.log.error(txt)
        self.terminate(1)
        
    def isConnected(self):
        '''Check if the communication channel is open
        
        :return: True, the communication channel is open ; False, otherwise.
        :rtype: bool
        '''
        return self._dcom.isOpen()
    
    def isInDevMode(self):
        '''Check if the application is executed in development mode
        
        :return: True, if the application is executed in development mode ; False, otherwise.
        :rtype: bool
        '''
        return self.config.dev_mode
        
    def getImageDir(self):
        '''Return the directory where the images are stored'''
        return IMG_DIR
    
    def getFirmwareDir(self):
        '''returns the directory where the firmware is stored'''
        return FIRM_DIR
    
    def getTempDir(self):
        '''return directory for temporary files'''
        return TEMP_DIR
    
    def getUserDataDir(self):
        '''returns the user data directory'''
        return USR_DATA_DIR
        
    @property
    def name(self):
        '''The application's name'''
        return 
            
    @property
    def dapi(self):
        '''The :class:`~dapi2.dapi2.DApi2` linked to application'''
        return self._dapi
    @property
    def board(self):
        '''The :class:`~dboard.base.BaseDBoard` linked to application'''
        return self._board
    @property
    def dcom(self):
        '''The :class:`~dapi2.dcom.base.BaseDCom` linked to application'''
        return self._dcom
    
    @property
    def tracer(self):
        '''The :class:`~dsmcp.app.tracer.Tracer` linked to application'''
        return self._tracer
    @property
    def firmwares(self):
        return self._firmwares



'''
Created on 27 sept. 2018

@author: fvoillat
'''

from PyQt5.QtWidgets import QDialog
import logging
from PyQt5.Qt import QIcon, QT_VERSION_STR, PYQT_VERSION_STR
import platform
import sys

from .ui_aboutdialog import Ui_AboutDialog


class AboutDialog(QDialog, Ui_AboutDialog):
    def __init__(self, parent):
        QDialog.__init__(self, parent)
        
        self._app = parent.app
        self.log = logging.getLogger(self.__class__.__name__)
        self.log.debug( 'Initialize' )
        self.setupUi(self)
        self.labelLogo.setPixmap(QIcon(':/img/icon_dcp.svg').pixmap(64,64) )
        
        
    def exec(self):
        #self.setWindowTitle()
        self.labelApplication.setText( self._app.applicationName() )
        self.labelVersion.setText( self._app.applicationVersion() )
        
        sys_info = '<html><body>'+\
            '<a href="https://www.python.org/">Python</a> : {} ({})'.format( ".".join([str(x) for x in sys.version_info[:3]]), platform.architecture()[0] ) + \
            '<br>OS : {}'.format(platform.platform()) + \
            '<br><a href="https://www.qt.io/">Qt</a> : {0!s} '.format(QT_VERSION_STR) + \
            '<a href="https://riverbankcomputing.com/software/pyqt/intro">PyQt</a> : {0!s}'.format(PYQT_VERSION_STR) + \
            '<br>Qt locale : `{0!s}` ; décimal: `{1!s}`'.format(self._app.locale.name(), self._app.locale.decimalPoint()) + \
            '<br><a href="https://github.com/dassym/PyDapi2">PyDapi2</a> : `{0!s}`'.format(self._app.dapi.version()) + \
            '</body></html>' 
 
        self.labelSystem.setText( sys_info )
        self.labelSystem.setOpenExternalLinks(True)
        return super().exec()
        



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/motormodedialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MotorModeDialog(object):
    def setupUi(self, MotorModeDialog):
        MotorModeDialog.setObjectName("MotorModeDialog")
        MotorModeDialog.resize(400, 313)
        self.verticalLayout = QtWidgets.QVBoxLayout(MotorModeDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBoxMode = QtWidgets.QGroupBox(MotorModeDialog)
        self.groupBoxMode.setObjectName("groupBoxMode")
        self.gridLayoutConfig = QtWidgets.QGridLayout(self.groupBoxMode)
        self.gridLayoutConfig.setObjectName("gridLayoutConfig")
        self.verticalLayout.addWidget(self.groupBoxMode)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setVerticalSpacing(2)
        self.gridLayout.setObjectName("gridLayout")
        self.registerAcceleration = QSpinBoxRegister(MotorModeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.registerAcceleration.sizePolicy().hasHeightForWidth())
        self.registerAcceleration.setSizePolicy(sizePolicy)
        self.registerAcceleration.setObjectName("registerAcceleration")
        self.gridLayout.addWidget(self.registerAcceleration, 0, 1, 1, 1)
        self.labeLightIntensity = QtWidgets.QLabel(MotorModeDialog)
        self.labeLightIntensity.setObjectName("labeLightIntensity")
        self.gridLayout.addWidget(self.labeLightIntensity, 1, 0, 1, 1)
        self.labelAcceleration = QtWidgets.QLabel(MotorModeDialog)
        self.labelAcceleration.setObjectName("labelAcceleration")
        self.gridLayout.addWidget(self.labelAcceleration, 0, 0, 1, 1)
        self.registerLightIntensity = QSpinBoxRegister(MotorModeDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.registerLightIntensity.sizePolicy().hasHeightForWidth())
        self.registerLightIntensity.setSizePolicy(sizePolicy)
        self.registerLightIntensity.setObjectName("registerLightIntensity")
        self.gridLayout.addWidget(self.registerLightIntensity, 1, 1, 1, 1)
        self.label = QtWidgets.QLabel(MotorModeDialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(MotorModeDialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 2, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(MotorModeDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(MotorModeDialog)
        self.buttonBox.accepted.connect(MotorModeDialog.accept)
        self.buttonBox.rejected.connect(MotorModeDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(MotorModeDialog)

    def retranslateUi(self, MotorModeDialog):
        _translate = QtCore.QCoreApplication.translate
        MotorModeDialog.setWindowTitle(_translate("MotorModeDialog", "Motor set points"))
        self.groupBoxMode.setTitle(_translate("MotorModeDialog", "Mode"))
        self.labeLightIntensity.setText(_translate("MotorModeDialog", "Light intensity"))
        self.labelAcceleration.setText(_translate("MotorModeDialog", "Acceleration"))
        self.label.setText(_translate("MotorModeDialog", "krpm/s"))
        self.label_2.setText(_translate("MotorModeDialog", "mA"))
from app.gui.widget.qspinboxregister import QSpinBoxRegister



# -*- coding: utf-8 -*-

from ._version import VERSION, __ver__, __version__, MAJOR_VERSION, MINOR_VERSION, REVISION_VERSION 

from .common import *


#
# exit_code = 0
# '''Application exit code''' 
#
# application = None
# '''The application itself'''


from .i18n import res_i18n  #@UnusedImport 

#from .launcher import prepareCfg, processCfg
from .base import BaseApp
from . import utils

#from .cli import *
#from .gui import *

#
# log = None
# '''The application logger'''
#
# args_config = {}
# '''The arguments given by INI file'''
# config = None
# '''The application configuration'''
#
# initial_dir = None
# '''The application initial directory. The folder of the main script.'''
#
# parser = None
# '''The argument line parser'''


'''

:author: fv
:date: Created on 12 mai 2021
'''
from PyQt5.Qt import QSpinBox

class QSpinBoxRegister(QSpinBox):
    '''
    classdocs
    '''


    def __init__(self, parent):
        '''
        Constructor
        '''
        QSpinBox.__init__(self, parent)
        self._reg = None
        self._board = None
        self._disable_events_cnt = 0
        self.valueChanged.connect(self.spinboxValueChanged)
        
    def onRegisterChange(self, reg, old_value=None, new_value=None):
        if not self.eventsEnabled: return
        self.blockSignals(True)
        try:
            if new_value is None:
                new_value = self._reg.value
            self.setValue(new_value)
        finally:
            self.blockSignals(False)
            
    def spinboxValueChanged(self, value):
        if not self.eventsEnabled: return 
        if self._reg is not None and self._board is not None:
            self.disableEvents()
            try:
                self._board.setReg(self._reg, value)
            finally:
                self.enableEvents()
    
    def setBoard(self, board):
        self._board = board    
        
    def setReg(self, reg=None ):
        if self._reg is not None:
            self._reg.disconnect(self.onRegisterChange)  
        self._reg = reg
        if self._reg is not None:
            self.setToolTip(self._reg.name+":"+self._reg.descr)
            self._reg.connect(self.onRegisterChange)
            self.setMinimum(self._reg.min)
            self.setMaximum(self._reg.max)
            self.onRegisterChange(self._reg, self._reg.value, self._reg.value)
        else:
            self.setToolTip('')
            
    def disableEvents(self):
        self._disable_events_cnt += 1
        return self._disable_events_cnt
    
    def enableEvents(self):
        if self._disable_events_cnt > 0:
            self._disable_events_cnt -= 1
        return self._disable_events_cnt            
        
    @property
    def board(self):
        return self._board
    @property
    def reg(self):
        return self._reg
    
    @property
    def eventsEnabled(self):
        return self._disable_events_cnt == 0  
    
        


'''
Created on 27 sept. 2018

@author: fvoillat
'''

from PyQt5.QtWidgets import QDialog
import logging

from .ui_aboutboarddialog import Ui_AboutBoardDialog 
from PyQt5.Qt import QIcon, QPixmap, Qt, QLabel
from app.gui.res import SystemModeConfigIcon
from dboard import SystemModeConfig

class AboutBoardDialog(QDialog, Ui_AboutBoardDialog):
    
    def __init__(self, parent):
        QDialog.__init__(self, parent)
        self.app = parent.app
        self._board = None
        self.log = logging.getLogger(self.__class__.__name__)
        self.log.debug( 'Initialize' )
        self.setupUi(self)
        
        self.labelFactoryDateIcon.setPixmap(QIcon(':/img/factory-date.svg').pixmap(32,32) )
        self.labelSnIcon.setPixmap(QIcon(':/img/sn.svg').pixmap(32,32))
        self.labelFirmwareIcon.setPixmap(QIcon(':/img/firmware.svg').pixmap(32,32))
        self._systemModeConfigLabels = {}
        
        for i, config in enumerate(SystemModeConfig):
            o = QLabel(self)
            if SystemModeConfigIcon[config] is not None:
                o.setPixmap(QIcon(SystemModeConfigIcon[config]).pixmap(48,48, QIcon.Disabled))
            else:
                o.setText(config.shortname)
            o.setToolTip("#{0:2d}:{1:s}".format(i,config.help))
            o.setObjectName('labelSystemModeConfig'+config.name)
            self.gridLayoutConfig.addWidget( o, i % 2, i // 2, 1, 1)
            self._systemModeConfigLabels[config] = o
        
        
    def _update(self):
        self.log.debug('_update')
        self.labelBoardName.setText(self._board.getName())
        data = self._board.getFactoryData()
        firm_version = self._board.getFirmwareVersion()
        firm_tag = self._board.getFirmwareTag()
        firm_date = self._board.getFirmwareDate()
        self.labelSerialNumber.setText("{0:05d}".format(data[0]))
        self.labelFactoryDate.setText(data[1].isoformat())
        firm = self.app.firmwares.get(
                        self._board,
                        firm_tag,
                        firm_version, 
                        firm_date
                        )
        if firm is not None:
            self.labelFirmware.setText('v<b>{sv[0]:d}.{sv[1]:02d}</b> – {f!s}'.format(sv=firm_version, f=firm))
        else:
            self.labelFirmware.setText("v<b>{sv[0]:d}.{sv[1]:02d}</b> – undefined {fd:s} 0x{ft:04x}".format(sv=firm_version, ft=firm_tag, fd=firm_date.isoformat()))

        rimg = ":/img/board/{0:s}.png".format(self._board.getName().lower())
        self.labelImage.setPixmap(
            QPixmap(rimg).scaled(self.labelImage.minimumSize(),  Qt.KeepAspectRatio, Qt.SmoothTransformation) 
            )


        cfg = self._board.getSystemModeConfiguration()
        self.groupBoxConfig.setTitle('Configuration (0x{0:04x})'.format(cfg.value))
        for i, config in enumerate(SystemModeConfig):
            o =  self._systemModeConfigLabels[config]
            if cfg & config != 0:
                if SystemModeConfigIcon[config] is not None:
                    o.setPixmap(QIcon(SystemModeConfigIcon[config]).pixmap(48,48, QIcon.Normal))
                else:
                    o.setText(config.shortname)
                o.setToolTip("#{0:2d}:{1:s}:YES".format(i,config.help))
            else:
                if SystemModeConfigIcon[config] is not None:
                    o.setPixmap(QIcon(SystemModeConfigIcon[config]).pixmap(48,48, QIcon.Disabled))
                else:
                    o.setText('¬'+config.shortname)
                o.setToolTip("#{0:2d}:{1:s}:NO".format(i,config.help))
        
        
    def setBoard(self, board):
        self._board = board
        self._update() 
    



'''
Created on 27 sept. 2018

@author: fvoillat
'''

from PyQt5.QtWidgets import QDialog
import logging
from functools import partial
from PyQt5.Qt import QIcon, QCheckBox, QSize

from .ui_motormodedialog import Ui_MotorModeDialog
from app.gui.res import SystemModeConfigIcon
from dboard import SystemModeConfig


class MotorModeDialog(QDialog, Ui_MotorModeDialog):
    
    def __init__(self, parent):
        QDialog.__init__(self, parent)
        self.app = parent.app
        self._board = None
        self.log = logging.getLogger(self.__class__.__name__)
        self.log.debug( 'Initialize' )
        self.setupUi(self)
        self.checkBoxConfig = {}
        
        for i, config in enumerate(SystemModeConfig):
            o = QCheckBox(config.descr, self)
            o.setIcon(QIcon(SystemModeConfigIcon[config]))
            o.setIconSize(QSize(48,48))
            o.setToolTip(config.help) 
            o.setObjectName('checkBoxConfig'+config.name)
            self.checkBoxConfig[config.name] = o
            o.stateChanged.connect(partial(self.onConfigStateChanged, config))
            self.gridLayoutConfig.addWidget( o, i % 4, i // 4, 1, 1)
            i += 1        
        
        
    def _update(self):
        self.log.debug('_update')
        
    def setBoard(self, board):
        self._board = board
        self.registerAcceleration.setBoard(self._board)
        self.registerAcceleration.setReg(self._board.regs.acr)
        self.registerLightIntensity.setBoard(self._board)
        self.registerLightIntensity.setReg(self._board.regs.lir)
        self._board.regs.smr.connect(self.onConfigChanged)
        
        self._update() 
        
        
    def onConfigStateChanged(self, config=None, checked=None):
        try:
            self.log.debug('onConfigStateChanged(config={0!s}, checked={1!s})'.format(config, checked))
            self._board.setBit(self._board.regs.smr(config.name.lower()), checked!=0)
        except Exception as e:
            self.parent.handleError(e)
            
    def onConfigChanged(self, reg, old, value):
        cfg = SystemModeConfig(value)
        for config in SystemModeConfig:
            self.checkBoxConfig[config.name].blockSignals(True)
            self.checkBoxConfig[config.name].setChecked(cfg & config != 0)
            self.checkBoxConfig[config.name].blockSignals(False)
    



from os.path import os
import sys
from enum import IntEnum
  

APP_NAME = 'dsmcp'
'''Application name'''

APP_DESCRIPTION = "The DSM-CP application offers a control panel to control Dassym boards."
'''Application description'''

DASSYM = 'Dassym'
'''Application editor'''

DASSYM_DOMAIN = 'dassym.com'
'''Application domain'''

TEMP_DIR = os.getenv('TMP') if os.name == 'nt' else '/tmp'
'''Path to temporary directory'''
#TODO: TEMP_DIR pour darwin 

DEFAULT_LOG_DIR = TEMP_DIR 
'''Path to deefault directory to store log file.'''

LOG_FORMAT = '%(asctime)s:%(levelname)s:%(name)s:%(message)s'
'''Format pattern for logging message'''


DIR = os.path.dirname(os.path.realpath(__file__))
'''Directory of application package'''

IMG_DIR = os.path.join( DIR, "..", "img")
'''Directory of application iamges'''

DATA_DIR = os.path.join( DIR, "..", "data")
'''Directory of application data'''

FIRM_DIR = os.path.join( DIR, "..", "firm")
'''Directory of Dassym's board firmware'''

USR_HOME_DIR = os.path.expanduser("~")
'''User home directory.'''


USR_DATA_DIR = os.path.join(USR_HOME_DIR, ".config","dassym") if sys.platform == 'linux' \
        else os.path.join(os.getenv('LOCALAPPDATA'), "Dassym") if sys.platform[:3] == 'win' \
        else os.path.join(USR_HOME_DIR, 'Library', 'Application Support', 'Dassym') if sys.platform[:6] == 'darwin' \
        else None
'''User Dassym directory.'''
        
USR_DATA_APP_DIR = os.path.join(USR_DATA_DIR, APP_NAME)
'''Application User Dassym directory.'''    

APP_PYTHON_REQUIRES = (3, 8)


class VERBOSITY(IntEnum):
    '''Enumeration for application verbosity level'''
    
    NONE = 0
    MINIMAL = 1
    LOW = 2
    MEDIUM = 3 
    HIGH = 4





'''
Created on 2 oct. 2020

@author: fv
'''

from PyQt5.QtCore import QCoreApplication



from ..base import BaseApp
import app

       
class BaseCliApp(BaseApp, QCoreApplication):
    '''Base class for CLI applications
    
    :param Namespace config: Application configuration
    '''

    NAME = 'dcp-base-cli'
    
    @classmethod
    def cast(cls, app):
        assert isinstance(app, BaseApp)
        app.__class__ = cls
        return app
    
    def __init__(self, app_dir):
        QCoreApplication.__init__(self, [] )
        self.setApplicationVersion(app.__version__)
        self.setApplicationName(self.NAME)
        self.setOrganizationName(app.DASSYM)
        self.setOrganizationDomain(app.DASSYM_DOMAIN)
        BaseApp.__init__(self, app_dir)

    def connectionCallback(self, options):
        optionDefault = options[0][0]
        for option in options:
            print("[{}] {}".format(option[0], option[1]))
        txt = input("Select option from options [{}]: ".format(optionDefault))
        
        if txt == "":
            return int(optionDefault)
        else:
            return int(txt)
        
    # def initialize(self):
        # BaseApp.initialize(self)
        
                
        
        
     
            
        
        



'''
Created on 14 oct. 2020

@author: fv
'''
from PyQt5.Qt import QPushButton

class QActionButton(QPushButton):
    '''
    classdocs
    '''


    def __init__(self, *args, **kwargs):
        QPushButton.__init__(self, *args, **kwargs)
        self._action = None
        if 'action' in kwargs:
            self.setAction(kwargs['action'])
            
    def _updateButtonStatus(self):
        if self._action is not None:
            self.setText(self._action.text())
            self.setStatusTip(self._action.statusTip())
            self.setToolTip(self._action.toolTip())
            self.setIcon(self._action.icon())
            self.setEnabled(self._action.isEnabled())
            self.setCheckable(self._action.isCheckable())
            self.setChecked(self._action.isChecked())
            
    def setAction(self, action):
        if self._action is not None:
            #TODO: reset action 
            pass 
        
        if action is not None:
            self._action = action
            self._updateButtonStatus()
            self._action.changed.connect(self._updateButtonStatus)
            self.clicked.connect( self._action.triggered )
            
            
            


'''Definition of main window for *basic* GUI application.

:author: F. Voillat
:date: 2021-0-24 Creation
:copyright: Dassym 2021
'''

from functools import partial

from app.gui.base import BaseMainWindow, BaseGuiApp
from .ui_basicwindow import Ui_BasicWindow  
from dboard.common import DBoardPreferedDapiMode
from app.gui.widget.qsvgpanel import ReverseKey, LightKey, IncrementKey, DecrementKey, MemoryKey, GearKey,\
    ParametersKey, WorkspaceKey, WorkspaceToggleKey
from app.gui.dlg.aboutboarddialog import AboutBoardDialog
from app.gui.dlg.sendfirmwaredialog import SendFirmwareDialog
from PyQt5.Qt import QDialog
from app.gui.dlg.flashprogressdialog import FlashProgressDialog
from dapi2.dapi2 import DApiAccessLevel
from app.gui.dlg.motormodedialog import MotorModeDialog
from .base import UserActionHandler, UserBoardActionHandler
    

class BasicWindow(BaseMainWindow, Ui_BasicWindow):
    '''Class of main window for *basic* GUI application.
    
    :param BaseGuiApp app: The application that owns the window
    ''' 
    def __init__(self, app, *args, **kwargs):
        '''Constructor'''
        BaseMainWindow.__init__(self, app, *args, **kwargs)
        Ui_BasicWindow.__init__(self, *args, **kwargs)
        self.setupUi(self)
    
    def _initWin(self):
        BaseMainWindow._initWin(self)
        
        self.pushButtonStart.setAction(self.actionMotorStart)
        self.actionMotorStart.triggered.connect(self.doMotorStart)
        self.pushButtonStop.setAction(self.actionMotorStop)
        self.actionMotorStop.triggered.connect(self.doMotorStop)
        self.actionLightEnabled.triggered.connect(self.doLightEnable)
        self.actionLightBlue.triggered.connect(self.doLightAlternate)
        self.actionMotorCCW.triggered.connect(self.doMotorReverse)
        self.actionAboutBoard.triggered.connect(self.doAboutBoard)
        self.actionAPISendFirmware.triggered.connect(self.doSendFirmware)
        self.actionMotorMode.triggered.connect(self.doShowMotorMode)
        
        self._addDialog('aboutboard', AboutBoardDialog(self))
        self._addDialog('motormode', MotorModeDialog(self))
        
        self.graphicsViewPanel.setSvg(self.app.getLayoutFile(self.app.config.layout))
        
        for key in self.graphicsViewPanel.keys:
            if isinstance(key, ReverseKey):
                key.pressShort.connect(self.doMotorReverse)
            elif isinstance(key, LightKey):
                key.pressShort.connect(self.doLightToggle)
                key.pressLong.connect(self.doLightAlternateToggle)
            elif isinstance(key, IncrementKey):
                key.pressShort.connect(self.doSpeedInc)
            elif isinstance(key, DecrementKey):
                key.pressShort.connect(self.doSpeedDec)
            elif isinstance(key, MemoryKey):
                key.pressShort.connect(partial(self.doRecallMem, key.num))
                key.pressLong.connect(partial(self.doStoreMem, key.num))
            elif isinstance(key, GearKey):
                key.pressShort.connect(partial(self.doSelectGear, key.numerator, key.denominator))
            elif isinstance(key, ParametersKey):
                key.pressShort.connect(self.doParameters)
            elif isinstance(key, WorkspaceToggleKey):
                key.pressShort.connect(partial(self.doWorkspaceToggle, key.num))
            elif isinstance(key, WorkspaceKey):
                key.pressShort.connect(partial(self.doWorkspace, key.num))
                
                
    def saveGeometries(self, settings):
        '''Saves the geometries of this window and its children windows
        
        :param settings: the data container.
        ''' 
        super().saveGeometries(settings)
        settings.beginGroup("Dialogs")
        for n, d in self.dialogs.items(): 
            settings.setValue(n+".geometry", d.saveGeometry())
            #settings.setValue(dlg.objectName()+".state", dlg.saveState())
        settings.endGroup()
        
    def restoreGeometries(self, settings):
        '''Restores the geometries of this window and its children windows
        
        :param settings: the data container.
        ''' 
        super().restoreGeometries(settings)
        settings.beginGroup("Dialogs")
        for n, d in self.dialogs.items():
            d.restoreGeometry(settings.value(n+".geometry"))
            #dlg.restoreState(settings.value(dlg.objectName()+"state"))
        settings.endGroup()
        
    def initialize(self):
        '''Initialize the window'''
        super().initialize()
        self.widgetSpeed.setBoard(self.board)
        self.graphicsViewPanel.setBoard(self.board)
        self.dialogs['motormode'].setBoard(self.board)
        
        
        
    def refresh(self):
        '''Refresh the windows according board setpoints'''
        super().refresh()
        
        def formatSpeed(speed):
            if speed >= 10:
                d = '0'
            elif speed >= 1:
                d = '1'
            else:
                d = '2'
            return ("{0:0."+d+"f}").format(speed)
            
        
        self.disableEvents()
        try:
            self.actionLightEnabled.setChecked(self.board.isLightEnabled())
            self.actionLightBlue.setChecked(self.board.isLightAlternate())
            self.actionMotorCCW.setChecked(self.board.isMotorReverse())
            self.actionMotorCW.setChecked(not self.board.isMotorReverse())
            self.widgetSpeed.setSpeed(self.board.motorSpeed())
            
            if self.board.isOnStandby():
                    self.label.setText('Standby')
                    self.graphicsViewPanel.setDisplayText('---')
            else:
                if self.board.isMotorRunning():
                    speed = self.board.motorRealSpeed()
                    self.graphicsViewPanel.setDisplayText(formatSpeed(speed/1000))
                    self.label.setText('Running:{0:d}'.format(speed))
                else:
                    speed = self.board.motorSpeed()
                    self.graphicsViewPanel.setDisplayText(formatSpeed(speed/1000))
                    self.label.setText('Stopped:{0:d}'.format(speed))
                
            self.graphicsViewPanel.initialize()    
            self.graphicsViewPanel.refresh()

            
        finally:
            self.enableEvents()        
        
        
    @UserBoardActionHandler
    def doMotorStart(self, checked=None):
        '''Handler for  *motor start* action'''
        self.board.motorStart()
                
    @UserBoardActionHandler
    def doMotorStop(self, checked=None):
        '''Handler for  *motor stop* action'''
        self.board.motorStop()

    @UserBoardActionHandler
    def doMotorReverse(self, checked=None):
        '''Handler for  *motor reverse* action'''
        self.board.motorReverse()
            
    
    def doLightToggle(self):
        self.doLightEnable(not self.board.isLightEnabled())
            
    @UserBoardActionHandler
    def doLightEnable(self, checked=None):
        '''Handler for  *light enable* action'''
        self.board.lightOn(checked)
            
    @UserBoardActionHandler
    def doSpeedInc(self):
        if self.board.motorSpeed() <= 2000:
            i = 100
        elif self.board.motorSpeed() < 10000:
            i = 1000
        else:
            i = 2000
        self.board.motorIncSpeed(i)
        
    @UserBoardActionHandler
    def doParameters(self):
        if self.dialogs['motormode'].isVisible():
            self.dialogs['motormode'].hide()
        else:
            self.dialogs['motormode'].show()
        
            
    @UserBoardActionHandler
    def doSpeedDec(self):
        if self.board.motorSpeed() <= 2000:
            i = 100
        elif self.board.motorSpeed() <= 10000:
            i = 1000
        else:
            i = 2000
        self.board.motorDecSpeed(i)
        
            
    @UserBoardActionHandler
    def doStoreMem(self, num):
        '''Handler for  *store memory* action'''
        self.board.memoryStore(num)
                 

    @UserBoardActionHandler
    def doWorkspace(self, num):
        '''Handler for  *Worspace* action'''
        self.app.board.setWorkspace(num)

    @UserBoardActionHandler
    def doWorkspaceToggle(self, num):
        '''Handler for  *Toggle Worspace* action'''
        if self.app.board.getWorkspace().standby:
            self.app.board.setWorkspace(num)
        else:
            self.app.board.setWorkspace(0)
            
            
    @UserBoardActionHandler
    def doRecallMem(self, num):
        '''Handler for  *recall memory* action'''
        self.board.memoryRecall(num)
        self.board.getRegisters('setpoints')
            
    @UserBoardActionHandler
    def doSelectGear(self, num, den):
        '''Handler for  *select gear ratio* action'''
        self.board.setGearRatio(num, den)

    def doLightAlternateToggle(self):
        '''Handler for  *toggle kind of light* action'''
        self.doLightAlternate(not self.board.isLightAlternate())

    @UserBoardActionHandler
    def doLightAlternate(self, checked=None):
        '''Handler for  *light alternate selection* action'''
        self.board.lightAlternate(checked)
            
    @UserActionHandler
    def doAboutBoard(self, checked=None):
        '''Handler for  *about board* action'''
        self._dialogs['aboutboard'].setBoard(self.board)
        self._dialogs['aboutboard'].exec()

    @UserActionHandler
    def doShowMotorMode(self, checked=None):
        '''Handler for  *Show motor mode dialog* action'''
        self.dialogs['motormode'].show()

    @UserBoardActionHandler
    def doSendFirmware(self, checked=None):
        '''Handler for  *send firmware* action'''
        dlg = SendFirmwareDialog(self)
        if dlg.exec() == QDialog.Accepted:
            temp = (self.app.autorefresh, self.app.board.getAccessLevel())
            self.app.autorefresh = False
            self.app.board.connect(DApiAccessLevel.SERVICE, DApiAccessLevel.SERVICE.passwd ) #@UndefinedVariable
            firmware = dlg.getFirmware()
            self.log.info('Firmware programming `{0!s}` '.format(firmware))
            self._flasProgressDialog = FlashProgressDialog(self,firmware)
            self._flasProgressDialog.show()
            self.app.processEvents()
            self.board.flashFirm(dlg.getFirmware(), self._flasProgressDialog.progress)
            self._flasProgressDialog.hide()
            del self._flasProgressDialog
            self.log.info('wait before reconnection ({}s)'.format(self.app.board.wait_after_reprogramming))
            self.app.sleep(self.app.board.wait_after_reprogramming)
            
            self.app.board.connect(temp[1], temp[1].passwd )
            self.app.autorefresh = temp[0]
        self.update()
    

class BasicGuiApp(BaseGuiApp):
    '''Class for *basic* GUI application
    
    :param Namespace config: Application configuration
    '''
    NAME = 'dcp-basic-gui'
    '''Application name'''

    def __init__(self, config):
        '''Constructor'''
        BaseGuiApp.__init__(self, config)
        
        
    def _initWin(self):
        self._window = BasicWindow(self) 
        self._window._initWin()
        
        
    def getAvailableActions(self, context={}):
        ret = super().getAvailableActions(context)
        if self._board is not None: 
            if self.isConnected():
                ret.add('actionAboutBoard')  
                ret.add('actionMotorMode')
                if self._board.isOnStandby():
                    ret.add('actionAPISendFirmware')
        return ret        
        
    def initialize(self):
        BaseGuiApp.initialize(self)
        if self.board.dmode == DBoardPreferedDapiMode.COMMAND:
            self.board.connect()
            

#        if self.board.isOnStandby():
#            self.board.setWorkspace(1)
        
         
    
        
        
        
        



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/aboutboarddialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutBoardDialog(object):
    def setupUi(self, AboutBoardDialog):
        AboutBoardDialog.setObjectName("AboutBoardDialog")
        AboutBoardDialog.resize(377, 203)
        AboutBoardDialog.setModal(True)
        self.gridLayout = QtWidgets.QGridLayout(AboutBoardDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.labelImage = QtWidgets.QLabel(AboutBoardDialog)
        self.labelImage.setMinimumSize(QtCore.QSize(128, 64))
        self.labelImage.setObjectName("labelImage")
        self.gridLayout.addWidget(self.labelImage, 0, 1, 1, 1)
        self.labelBoardName = QtWidgets.QLabel(AboutBoardDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelBoardName.sizePolicy().hasHeightForWidth())
        self.labelBoardName.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.labelBoardName.setFont(font)
        self.labelBoardName.setTextFormat(QtCore.Qt.RichText)
        self.labelBoardName.setObjectName("labelBoardName")
        self.gridLayout.addWidget(self.labelBoardName, 0, 0, 1, 1)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.labelSnIcon = QtWidgets.QLabel(AboutBoardDialog)
        self.labelSnIcon.setObjectName("labelSnIcon")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.labelSnIcon)
        self.labelSerialNumber = QtWidgets.QLabel(AboutBoardDialog)
        self.labelSerialNumber.setTextFormat(QtCore.Qt.RichText)
        self.labelSerialNumber.setObjectName("labelSerialNumber")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.labelSerialNumber)
        self.labelFactoryDateIcon = QtWidgets.QLabel(AboutBoardDialog)
        self.labelFactoryDateIcon.setObjectName("labelFactoryDateIcon")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.labelFactoryDateIcon)
        self.labelFactoryDate = QtWidgets.QLabel(AboutBoardDialog)
        self.labelFactoryDate.setTextFormat(QtCore.Qt.RichText)
        self.labelFactoryDate.setObjectName("labelFactoryDate")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.labelFactoryDate)
        self.labelFirmwareIcon = QtWidgets.QLabel(AboutBoardDialog)
        self.labelFirmwareIcon.setObjectName("labelFirmwareIcon")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.labelFirmwareIcon)
        self.labelFirmware = QtWidgets.QLabel(AboutBoardDialog)
        self.labelFirmware.setTextFormat(QtCore.Qt.RichText)
        self.labelFirmware.setObjectName("labelFirmware")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.labelFirmware)
        self.gridLayout.addLayout(self.formLayout, 1, 0, 1, 2)
        self.buttonBox = QtWidgets.QDialogButtonBox(AboutBoardDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Vertical)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 0, 2, 1, 1)
        self.groupBoxConfig = QtWidgets.QGroupBox(AboutBoardDialog)
        self.groupBoxConfig.setObjectName("groupBoxConfig")
        self.gridLayoutConfig = QtWidgets.QGridLayout(self.groupBoxConfig)
        self.gridLayoutConfig.setObjectName("gridLayoutConfig")
        self.gridLayout.addWidget(self.groupBoxConfig, 2, 0, 1, 2)

        self.retranslateUi(AboutBoardDialog)
        self.buttonBox.accepted.connect(AboutBoardDialog.accept)
        self.buttonBox.rejected.connect(AboutBoardDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(AboutBoardDialog)

    def retranslateUi(self, AboutBoardDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutBoardDialog.setWindowTitle(_translate("AboutBoardDialog", "About board"))
        self.labelImage.setText(_translate("AboutBoardDialog", "BOARD"))
        self.labelBoardName.setText(_translate("AboutBoardDialog", "MB-xx"))
        self.labelSnIcon.setText(_translate("AboutBoardDialog", "SN"))
        self.labelSerialNumber.setText(_translate("AboutBoardDialog", "00000"))
        self.labelFactoryDateIcon.setText(_translate("AboutBoardDialog", "Date"))
        self.labelFactoryDate.setText(_translate("AboutBoardDialog", "0000-00-00"))
        self.labelFirmwareIcon.setText(_translate("AboutBoardDialog", "Firmware"))
        self.labelFirmware.setText(_translate("AboutBoardDialog", "TextLabel"))
        self.groupBoxConfig.setTitle(_translate("AboutBoardDialog", "Configuration"))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/sendfirmwaredialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SendFirmwareDialog(object):
    def setupUi(self, SendFirmwareDialog):
        SendFirmwareDialog.setObjectName("SendFirmwareDialog")
        SendFirmwareDialog.resize(407, 233)
        SendFirmwareDialog.setModal(True)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(SendFirmwareDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.frame = QtWidgets.QFrame(SendFirmwareDialog)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.labelActual = QtWidgets.QLabel(self.frame)
        font = QtGui.QFont()
        font.setItalic(True)
        self.labelActual.setFont(font)
        self.labelActual.setObjectName("labelActual")
        self.verticalLayout.addWidget(self.labelActual)
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.comboBoxFirmware = QtWidgets.QComboBox(self.frame)
        self.comboBoxFirmware.setObjectName("comboBoxFirmware")
        self.verticalLayout.addWidget(self.comboBoxFirmware)
        self.groupBoxAboutFirmware = QtWidgets.QGroupBox(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBoxAboutFirmware.sizePolicy().hasHeightForWidth())
        self.groupBoxAboutFirmware.setSizePolicy(sizePolicy)
        self.groupBoxAboutFirmware.setObjectName("groupBoxAboutFirmware")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBoxAboutFirmware)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.labelFirmwareDesc = QtWidgets.QLabel(self.groupBoxAboutFirmware)
        self.labelFirmwareDesc.setObjectName("labelFirmwareDesc")
        self.verticalLayout_3.addWidget(self.labelFirmwareDesc)
        self.verticalLayout.addWidget(self.groupBoxAboutFirmware)
        self.verticalLayout_2.addWidget(self.frame)
        self.buttonBox = QtWidgets.QDialogButtonBox(SendFirmwareDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok|QtWidgets.QDialogButtonBox.Reset)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(SendFirmwareDialog)
        self.buttonBox.accepted.connect(SendFirmwareDialog.accept)
        self.buttonBox.rejected.connect(SendFirmwareDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SendFirmwareDialog)

    def retranslateUi(self, SendFirmwareDialog):
        _translate = QtCore.QCoreApplication.translate
        SendFirmwareDialog.setWindowTitle(_translate("SendFirmwareDialog", "DSMCP - Programming"))
        self.labelActual.setText(_translate("SendFirmwareDialog", "TextLabel"))
        self.label.setText(_translate("SendFirmwareDialog", "Firmware:"))
        self.groupBoxAboutFirmware.setTitle(_translate("SendFirmwareDialog", "GroupBox"))
        self.labelFirmwareDesc.setText(_translate("SendFirmwareDialog", "labelFirmwareDesc"))



'''
Created on 1 avr. 2020

@author: fv
'''


from PyQt5.Qt import QFrame, QGridLayout, QSlider, QProgressBar, QSpinBox, Qt, QSizePolicy,\
    QLabel, QIcon, QSize

class QSpeed(QFrame):
    '''
    classdocs
    '''

    def __init__(self, parent, board=None, graph=None):
        '''
        Constructor
        '''
        self._board = None
        self._graph = graph
        super().__init__(parent)
        self._layout = QGridLayout()
        
        self._slider = QSlider(Qt.Horizontal, self)
        self._slider.setMaximum(40000)
        self._slider.setPageStep(1000)
        self._slider.setObjectName('sliderMotorSpeed')
        self._icon = QLabel('Speed', self)
        self._icon.setPixmap(QIcon(':/img/speed.svg').pixmap(QSize(64,64)))
        self._icon.setMinimumSize(64, 64)
        self._icon.setSizePolicy(QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum))
        self._progressBar = QProgressBar(self)
        self._progressBar.setMaximum(40000)
        self._progressBar.setObjectName('progressBarMotorSpeed')
        self._progressBar.setFormat('%v')
        self._spinBox = QSpinBox(self)
        self._spinBox.setMaximum(40000)
        self._spinBox.setSingleStep(1000)
        self._spinBox.setObjectName('spinBoxMotorSpeed')
        sizePolicy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self._spinBox.sizePolicy().hasHeightForWidth())
        self._spinBox.setSizePolicy(sizePolicy)
        
        self._layout.addWidget(self._icon, 0, 0, 2, 1)
        self._layout.addWidget(self._slider, 0, 1, 1, 1)
        self._layout.addWidget(self._progressBar, 1, 1, 1, 1)
        self._layout.addWidget(self._spinBox, 0, 2, 2, 1)

        self.setGraph(graph)
        
        self.setLayout(self._layout)        
        
        self._slider.valueChanged.connect(self._spinBox.setValue)
        self._spinBox.valueChanged.connect(self.onSpinboxValueChanged)
        
        self.setBoard(board)
        
    def onRealSpeedChanged(self, reg, old=None, value=None):
        self._progressBar.setValue(value)
        
    
    def onSpeedChanged(self, reg, old=None, value=None):
        self._spinBox.blockSignals(True)
        self._spinBox.setValue(value)
        self._slider.setValue(value)
        self._spinBox.blockSignals(False)
        
    def onSpinboxValueChanged(self, value):
        self._slider.setValue(value)
        self._board.setMotorSpeed(value)
        
    def setBoard(self, board=None):
        if self._board is not None:
            pass  
        self._progressBar.setValue(0)
        self._spinBox.blockSignals(True)
        self._spinBox.setValue(0)
        self._slider.setValue(0)
        self._board = board
        if self._board is not None:
            self.setRange(self._board.speed_range)
            self._board.regs.msr.connect(self.onRealSpeedChanged)
            self._board.regs.scr.connect(self.onSpeedChanged)
        self._spinBox.blockSignals(False)
        
    def setSpeed(self, value):
        self.onSpeedChanged(None, value=value)

    def setGraph(self, graph):
        self._graph = graph
        if self._graph:
            self._layout.addWidget(self._graph, 2, 0, 1, 3)
            #self._graph.draw_idle()
            
    def setRange(self, valuerange):
        self._spinBox.setMaximum(valuerange.upper)
        self._slider.setMaximum(valuerange.upper)
        self._progressBar.setMaximum(valuerange.upper)
        if self._graph:
            self._graph.set_ylim(0,valuerange.upper)
         
            
    def update(self):
        if self._graph:
                self._graph.addData(self._motor.get_real_speed())
    
    @property
    def graph(self):
        return self._graph      
    @property
    def value(self):
        return self._spinBox.value()
        


'''

:author: fv
:date: Created on 24 mars 2021
'''
from app.gui.base import BaseGuiApp

class DevGuiApp(BaseGuiApp):
    '''Class for *development* GUI application
    
    :param Namespace config: Application configuration
    '''
    NAME = 'dcp-dev'

    def __init__(self, config):
        '''Constructor'''
        super().__init__(config)


'''Definition of *basic* CLI application.

:author: F. Voillat
:date: 2021-0-24 Creation
:copyright: Dassym 2021
'''
from app.cli.base import BaseCliApp
from app.cli.dcpshell import DcpShell
from app.base import BaseApp
from dapi2.dapi2 import DApiAccessLevel


class BasicCliApp(BaseCliApp):
    '''Class for *basic* CLI application
    
    :param Namespace config: Application configuration
    '''
    NAME = 'dcp-basic'
    '''Application name'''
    


    def __init__(self, app_dir):
        '''Constructor'''
        super().__init__(app_dir)
        

        
    def initialize(self):
        '''Initialize application'''
        super().initialize()
        self.shell = DcpShell(self)
    
    def run(self):
        '''Executes application'''
        self.log.info(BaseApp.tr('Run single shot'))
        level = self.board.getAccessLevel() 
        if level != DApiAccessLevel.NO:
            self.shell.prompt = level.name[0]+">"
        else: 
            self.shell.prompt = ">"
        self.shell.cmdloop(BaseApp.tr('Welcome to \033[1m{0:s}\033[0m version {1:s} shell! Type `help` or `?` to list commands.').format(
                    self.applicationName(),
                    self.applicationVersion()
                    ))
        return self.terminate()
        


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/selectconnectiondialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SelectionConnectionDialog(object):
    def setupUi(self, SelectionConnectionDialog):
        SelectionConnectionDialog.setObjectName("SelectionConnectionDialog")
        SelectionConnectionDialog.resize(461, 103)
        SelectionConnectionDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(SelectionConnectionDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(SelectionConnectionDialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.comboBox = QtWidgets.QComboBox(SelectionConnectionDialog)
        self.comboBox.setObjectName("comboBox")
        self.verticalLayout.addWidget(self.comboBox)
        spacerItem = QtWidgets.QSpacerItem(20, 27, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.buttonBox = QtWidgets.QDialogButtonBox(SelectionConnectionDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(SelectionConnectionDialog)
        self.buttonBox.accepted.connect(SelectionConnectionDialog.accept)
        self.buttonBox.rejected.connect(SelectionConnectionDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(SelectionConnectionDialog)

    def retranslateUi(self, SelectionConnectionDialog):
        _translate = QtCore.QCoreApplication.translate
        SelectionConnectionDialog.setWindowTitle(_translate("SelectionConnectionDialog", "Dsmcp - Select remote connection"))
        self.label.setText(_translate("SelectionConnectionDialog", "Select remote connection"))



'''

:author: fv
:date: Created on 19 mars 2021
'''


import argparse
import configparser
import os

from .common import APP_NAME
from ._version import __version__
from .base import BaseApp



class Launcher(object):
    '''class used to launch the application
    
    :param str app_dir: The application root directory.
    '''
    def __init__(self, app_dir, add_help=False):
        
        self.app_dir = app_dir
        self.add_help=add_help
        self.args_config = {}
        self.parser = None
        self._parser0 = None
        self._initCfg()
        
    
    def _initCfg(self):
        self._parser0 = argparse.ArgumentParser(description='{0:s} (v{1:s})'.format(APP_NAME, __version__), add_help=self.add_help)
        self._parser0.add_argument("-c", "--config", dest="config", help = BaseApp.tr("Configuration file (.ini)") )
        self._parser0.add_argument("--no-gui", dest="no_gui", action='store_true', default=False, help = BaseApp.tr("If present start no GUI application") )  
        self._parser0.add_argument("--qt5-options", dest="qt5_options", help = BaseApp.tr("Qt5 options") )
        self._parser0.add_argument("--lang", dest="lang", help=BaseApp.tr("ISO name of language to use."))
        verbosity_grp = self._parser0.add_mutually_exclusive_group()
        verbosity_grp.add_argument("-v", dest="verbosity_cnt", action="count", help=BaseApp.tr("Verbosity level (default=none)."))
        verbosity_grp.add_argument("--verbosity", dest="verbosity", help=BaseApp.tr("Verbosity level (default=0:none)."), default=0)

        
    def preConfig(self):
        '''Pre-configuration of application with the already defined arguments''' 
        self.config, remaining_argv0 = self._parser0.parse_known_args() #@UnusedVariable
        
        if self.config.config is not None:
            
            if os.path.dirname(self.config.config) == '':
                self.config.config = os.path.join(self.app_dir, self.config.config )
            
            if not os.path.exists(self.config.config):
                raise Exception(BaseApp.tr("Configuration file not found!"))
            cfg = configparser.ConfigParser()
            cfg.read(self.config.config,encoding='utf_8_sig')  
    
            for section in cfg.sections() :
                if section == 'GLOBAL':
                    prefix = ''
                else:
                    prefix = section.lower()+'_'
                for k in cfg[section]:
                    v = cfg[section][k]
                    if v.lower() in ('yes', 'true'):
                        v = True
                    elif v.lower() in ('no', 'false'):
                        v = False
                    
                    self.args_config[prefix+k] = v
                    
        self.parser = argparse.ArgumentParser( parents=[self._parser0], add_help=not self.add_help)
        
        
    def launch(self, app_class):
        '''Create and launch the application
        
        :param BaseApp app_class: The applications's class to create an launch.
        :return: The application
        '''
        app = app_class(self.app_dir)
        app.prepareL10n(self.config.lang)
        app.prepareCfg(self.parser)
        app.processCfg(self.parser, self.args_config)
        app.initLog()
        return app
    
    @property
    def parser0(self):
        return self._parser0
    
#===============================================================================
# 
# 
# def prepareCfg():
#     #global log, args_config, config, initial_dir
#     
#     parser0 = argparse.ArgumentParser(description='{0:s} (v{1:s})'.format(app.APP_NAME, app.__version__), add_help=False)
#     parser0.add_argument("-c", "--config", dest="config", help = tr("Configuration file (.ini)") )
#     parser0.add_argument("--no-gui", dest="no_gui", action='store_true', default=False, help = tr("If present start no GUI application") )  
#     parser0.add_argument("--qt5-options", dest="qt5_options", help = tr("Qt5 options") )
#     parser0.add_argument("--lang", dest="lang", help=tr("ISO name of language to use."))
#     parser0.add_argument("-v", "--verbosity", action="count", help=tr("Verbosity level (default=0)."), default=0)
#     
#     app.config, remaining_argv0 = parser0.parse_known_args() #@UnusedVariable
#     
#     if app.config.config is not None:
#         
#         if os.path.dirname(app.config.config) == '':
#             app.config.config = os.path.join(app.initial_dir, app.config.config )
#         
#         if not os.path.exists(app.config.config):
#             raise Exception(tr("Configuration file not found!"))
#         cfg = configparser.ConfigParser()
#         cfg.read(app.config.config,encoding='utf_8_sig')  
# 
#         for section in cfg.sections() :
#             if section == 'GLOBAL':
#                 prefix = ''
#             else:
#                 prefix = section.lower()+'_'
#             for k in cfg[section]:
#                 v = cfg[section][k]
#                 if v.lower() in ('yes', 'true'):
#                     v = True
#                 elif v.lower() in ('no', 'false'):
#                     v = False
#                 
#                 app.args_config[prefix+k] = v
# 
#     app.parser = argparse.ArgumentParser( parents=[parser0] )
#     
#     app.parser.add_argument("-l", "--log-level", dest="loglevel", choices=[ 'error', 'warning', 'debug', 'info', 'noset'], help=tr("Log level. (default = info)"), default="info"   )
#     app.parser.add_argument("--log-file", dest="logfile", default='stderr', help=tr("LOG file (default = stderr)") )
#     app.parser.add_argument("--log-max-size", dest="log_maxbytes", help=tr("Maximum LOG file size in bytes (default=100000)."), default=100000)
#     app.parser.add_argument("--log-backup-count", dest="log_backupcount", help=tr("Number of LOG files retained (default=10)."), default=10)
#     app.parser.add_argument("-t", "--trace", dest="trace", action='store_true', help=tr("If present, trace API message exchanges (default = not)"), default=False )
#     
#     
#     dcom_grp = app.parser.add_mutually_exclusive_group(required=True)
#     dcom_grp.add_argument("-S", "--serial", dest="serial", help=tr("Serial port for direct control."), metavar="SERIAL")
#     dcom_grp.add_argument("-H", "--host", dest="host", help=tr("Host name and port for remote control. (host:port)"), metavar="HOST")
#     
#     app.parser.add_argument("-C", "--command-mode", dest="command_mode", action='store_true', help=tr("Sets the preferred DAPI2 mode to 'command'. (Default=command mode)"),default=True )
#     app.parser.add_argument("-R", "--register-mode", dest="command_mode", action='store_false', help=tr("Sets the preferred DAPI2 mode to 'register'. (Default=command mode)"))
#     app.parser.add_argument("-D", "--dev-mode", dest="dev_mode", action='store_true', help=tr("Sets DSMCP execution mode 'development'. (Default=normal mode)"))
# 
#     app.parser.add_argument("-b", "--baudrate", dest="baudrate", type=int, help=tr("Baud rate. (default: {0})".format(dapi2.COM_SPEEDS[0]) ), metavar="BAUDRATE", default=dapi2.COM_SPEEDS[0])
#     app.parser.add_argument("-v", "--version", action='version',  help=tr("Show software version"), version='%(prog)s version {0:s}'.format(app.__version__)   )
#     app.parser.add_argument("--firm-paths", dest="firm_paths", help=tr("Search paths for firmware binaries. The different paths are separated by a semicolon (;)"))
#     app.parser.add_argument("--registers",  help=tr("The XML registers definition file"), dest="regfile",  default="data:dapi-registers.xml" )
#     app.parser.add_argument("--layout",  help=tr("The GUI layout name or SVG file. (default: vantage)"), dest="layout", default="vantage")
#     
#     
# def processCfg():   
#     
#     app.parser.set_defaults(**app.args_config)
#     app.config = app.parser.parse_args()
#     
#     
#     if app.config.loglevel:
#         app.config.loglevel = getattr(logging, app.config.loglevel.upper())
#     
#     app.log = logging.getLogger()
#     
#     if app.config.logfile is None or app.config.logfile == 'stderr':
#         log_handler = logging.StreamHandler(sys.stderr)
#     elif app.config.logfile == 'stdout':
#         log_handler = logging.StreamHandler(sys.stdout)
#     else:
#         if os.path.dirname(app.config.logfile) != '':
#             logfile = os.path.normpath(app.config.logfile)
#         else:
#             logfile = os.path.normpath(os.path.join(app.DEFAULT_LOG_DIR, app.config.logfile))
#             
#         log_handler = logging.handlers.RotatingFileHandler( filename=logfile, maxBytes=app.config.log_maxbytes, backupCount=app.config.log_backupcount)
# 
#     if app.config.verbosity:
#             try:
#                 if isinstance(app.config.verbosity, int):
#                     app.config.verbosity = VERBOSITY(app.config.verbosity)
#                 elif app.config.verbosity.isdigit():
#                     app.config.verbosity = VERBOSITY(str(app.config.verbosity))
#                 else:
#                     app.config.verbosity = VERBOSITY[app.config.verbosity]
#             except:
#                 app.config.verbosity = VERBOSITY.NONE
#     
#     
#     if 'firm_paths' in app.config and app.config.firm_paths is not None:
#         app.config.firm_paths = app.config.firm_paths.split(';') 
#     else:
#         app.config.firm_paths = []
#         
# 
# 
# 
#     app.log.setLevel(app.config.loglevel)
#     logfmt = logging.Formatter(app.LOG_FORMAT)
#     log_handler.setFormatter(logfmt)   
#     app.log.addHandler(log_handler)
#===============================================================================
    

# def initialize():
    # assert app.application is None
    # try:
        # app.log.info(tr('Start {}  v{}').format(app.NAME, app.VERSION))
        # app.log.debug(tr('Python version: {} ({})').format( ".".join([str(x) for x in sys.version_info[:3]]) , platform.architecture()[0]))
        # app.log.debug(tr('OS: {}').format(platform.platform()))
        # app.log.debug(tr('Configuration:\n\t') + app.utils.configStr(app.config)  )
        #
        # if app.config.no_gui:
            # app.application = BaseCliApp(app.config)
        # else:
            # raise Exception("GUI not yet implemanted!")
            # #TODO:app.application = GuiApp(config) 
            #
        # app.application.initialize()
        #
    # except Exception as e:
        # handleException(e)
        #
# def run():
    # assert app.application is not None
    # try:
        # app.log.info( tr('Run {}').format(app.NAME) )
        # return app.application.run()
    # except Exception as e:
        # handleException(e)
        #
# def terminate():
    #
    # assert app.application is not None
    #
    # app.application.terminate()
    #
    # try:
        # app.log.info( tr('End {0!s} ({1:d}).').format(app.NAME, app.exit_code) )
    # except Exception as e:
        # handleException(e)
    
# def handleException(e):
    # if app.log:
        # app.log.error(tr('Error*:')+str(e))
    # else:
        # print(tr('Error*:')+str(e), file=sys.stderr)
    # if app.config.loglevel == logging.DEBUG:
        # s = traceback.format_exc()
        # if s and app.log:
            # app.log.error(tr('Trace\n')+ str(s))
        # else:
            # print(tr('Trace\n')+ str(s), file=sys.stderr)
    # ## Boîte de dialogue pour l'affichage de'erreur
    # if isinstance(app.application, QApplication):
        # dialog = QMessageBox(QMessageBox.Critical, app.NAME+tr('Error:'), tr("An error has occurred!\nDetail:\n")+str(e), QMessageBox.Ok )
        # dialog.exec()
    # if app.exit_code == 0:
        # app.exit_code=9
        #
    # sys.exit(app.exit_code)




'''Launcher for *dsmcp basic*.

:author: F. Voilat
:date: Created on 19 mars 2021
'''

import sys
from os.path import os

if __name__ == '__main__':
    if sys.version_info < (3,8):
        raise Exception("DSMCP requires at least Python version 3.8")
    
    from app.launcher import Launcher
    
    launcher = Launcher(os.path.dirname(__file__))
    launcher.preConfig()
    if launcher.config.no_gui:
        from app.cli import BasicCliApp
        application = launcher.launch(BasicCliApp) 
    else:
        from app.gui import BasicGuiApp
        application = launcher.launch(BasicGuiApp)  
    try: 
        application.initialize()
        application.run()
    except Exception as e:
        application.handleError(e)
        sys.exit(1)


'''

:author: fv
:date: Created on 19 mars 2021
'''
import re as RE

import chardet
import os.path
from PyQt5.QtCore import QCoreApplication




MAX_RANGE_SIZE = 2000
SN_LASTNUM_RE = RE.compile(r'(?:[^\d]*(\d+)[^\d]*)+')

def tr(text, disambiguation=None, n=-1):
    return QCoreApplication.translate('@default',text, disambiguation, n)


def configStr(config, sep='\n\t'):
    l = [ str(x) +" = "+ ( str(config.__dict__[x]) if config.__dict__[x] != None else '<None>' ) + ' ('+type(config.__dict__[x]).__name__+')'   for x in sorted(config.__dict__.keys())  ]
    return sep.join( l )

def getXmlName(xelement, lang='en'):
    xdescr = xelement.find( "name[@lang='{}']".format(lang)  )
    if xdescr is None:
        xdescr = xelement.find( "name[1]"  )
    if xdescr is not None:
        return xdescr.text
    return None


def getXmlDescr(xelement, lang='en'):
    xdescr = xelement.find( "descr[@lang='{}']".format(lang)  )
    if xdescr is None:
        xdescr = xelement.find( "descr[1]"  )
    if xdescr is not None:
        return xdescr.text
    return None

def str2version(text):
    if text is not None and len(text)>0:
        m = RE.search(r'\D*(\d+)\.(\d{2,})', text)
        if m==None:
            raise Exception("Invalid version : {0!s}".format(text))  
        return  ( int(m.group(1)) , int(m.group(2)) )
    else:
        return None
    
def version2str(version):
    try:
        return "{0:d}.{1:02d}".format(version[0],version[1])
    except TypeError:
        return None

def str2bool(text):
    return text is not None and len(text)>0 and text.lower()[0] in ('y', 't', '1', 'o')


def increment(s, inc=1):
    '''Recherche la dernière séquence de digits d'une chaîne est l'incrémente.
    :param str s: Chaîne de caractères à incrémenter de inc.
    :praam int inc: Valeur de l'incrément. Par défaut = 1.
    '''
    m = SN_LASTNUM_RE.search(s)
    if m:
        n = str(int(m.group(1))+inc)
        start, end = m.span(1)
        s = s[:max(end-len(n), start)] + n + s[end:]
    return s


def genrateSnList( start, n, inc=1):
    ret = set()
    for i in range(int(n)): #@UnusedVariable
        ret.add( start )
        start = increment(start, inc)
    return ret

def snFromRange(r, inc=1):
    '''
    Retourne une liste de numéros de série depuis une chaîne de caractères au format A-B ou A:C. Où A et B sont les bornes de la plage, C le nombre de SN à produire.
    :param str r: Plage de SN au format A-B ou A:C. A=SN de début, B=SN de fin, C=nombre de SN.
    :param int inc:  Valeur de l'incréement. Par défaut = 1.
    :return: Une liste de SN [A, A+inc, A+2*inc, ..., B]
    '''
    ret = set()
    if ':' in r:
        (start,count)=r.split(':',2)
        for i in range(int(count)):
            ret.add( start )
            start = increment(start, inc)
    elif '-' in r:
        (start,end)=r.split('-',2)
        ret.add( start.strip() )
        i = 0
        end = end.strip()            
        if end < start :
            end, start = start, end
        
        last = start
        while True:
            n = increment( last, inc )
            last = n
            if n <= end :
                ret.add(n)
                i += 1
                if i > MAX_RANGE_SIZE:
                    raise Exception('La plage de SN {} comporte plus de {} numéros!'.format(r,MAX_RANGE_SIZE))
            else:
                break
    else:
        ret.add(r)
    return ret

def snFromArg(arg, inc=1, exclude=[]):
    '''
    :param str arg: Argument de la ligne de commande au format A-B ou A:C. Où A et B sont les bornes de la plage, C le nombre de SN à produire.
    :param int inc: Valeur de l'incréement. Par défaut = 1. 
    :return: Une liste de SN [A, A+inc, A+2*inc, ..., B]
    '''

    ret = set()
    
    for r0 in arg:
        for r1 in r0.split(';'):
            ret |= snFromRange(r1, inc)
            
    
    for r0 in exclude:
        for r1 in r0.split(';'):
            ret -= snFromRange(r1, inc)
    
    return ret


def snFromFile(fpath, inc=1):
    global LOG
    ret = set()
    raw = open(fpath, 'rb').read(min(32, os.path.getsize(fpath)))
    result = chardet.detect(raw)
    encoding = result['encoding']
    if encoding in ('UTF-16BE', 'UTF-16LE') :
        raise Exception( 'Encodage UTF-16 non supporté !' ) 
    with open(fpath, 'r', encoding=encoding) as f:
        for sn in f:
            sn = sn.strip()            
            sn = snFromRange(sn, inc)
            ret |= sn
    return ret

def compactSnList( sn_list ):
    '''
    Générateur d'une liste de tuples représentant toutes les plages d'une liste de numéros de série.
    @param sn_list Liste des numéros de série [a, b, c, ...].
    @return Liste de tuples [ (<from> ,<to>), ... ] 
    '''
    sn_list = sorted(sn_list)
    i = 0
    for j in range(1,len(sn_list)):
        if sn_list[j] > increment(sn_list[j-1]):
            yield (sn_list[i],sn_list[j-1])
            i = j
    yield (sn_list[i], sn_list[-1])
    
    
def compactSnListToStr(sn_list):
    s = []
    for x in sn_list:
        if x[0] == x[1]:
            s.append( x[0] )
        elif increment(x[0]) == x[1]:
            s.append( x[0] + ',' + x[1] )
        else:
            s.append( x[0] + '-' + x[1] )
    return ';'.join( s )




import datetime as DT

MAJOR_VERSION = 0
'''Major version number'''

MINOR_VERSION = 1
'''Minor version number'''

REVISION_VERSION = 1
'''Revision version number'''

VERSION = (MAJOR_VERSION, MINOR_VERSION, REVISION_VERSION)
'''Version number as 3-tuples containing *major*, *minor* and *revision* numbers.''' 

__version__ = '{0:d}.{1:d}.{2:d}'.format(*VERSION)
'''Appication version number'''

__ver__ = '{0:d}.{1:d}'.format(*VERSION)
'''Appication short version number'''

DATE = DT.date(2021, 5, 25)


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/flashprogressdiaog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_FlashProgressDialog(object):
    def setupUi(self, FlashProgressDialog):
        FlashProgressDialog.setObjectName("FlashProgressDialog")
        FlashProgressDialog.resize(400, 141)
        FlashProgressDialog.setModal(False)
        self.gridLayout = QtWidgets.QGridLayout(FlashProgressDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.progressBar = QtWidgets.QProgressBar(FlashProgressDialog)
        self.progressBar.setProperty("value", 24)
        self.progressBar.setObjectName("progressBar")
        self.gridLayout.addWidget(self.progressBar, 2, 0, 1, 2)
        self.labelIcon = QtWidgets.QLabel(FlashProgressDialog)
        self.labelIcon.setObjectName("labelIcon")
        self.gridLayout.addWidget(self.labelIcon, 0, 0, 1, 1)
        self.labelText = QtWidgets.QLabel(FlashProgressDialog)
        self.labelText.setObjectName("labelText")
        self.gridLayout.addWidget(self.labelText, 0, 1, 1, 1)
        self.labelRemainingTime = QtWidgets.QLabel(FlashProgressDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelRemainingTime.sizePolicy().hasHeightForWidth())
        self.labelRemainingTime.setSizePolicy(sizePolicy)
        self.labelRemainingTime.setObjectName("labelRemainingTime")
        self.gridLayout.addWidget(self.labelRemainingTime, 1, 0, 1, 2)

        self.retranslateUi(FlashProgressDialog)
        QtCore.QMetaObject.connectSlotsByName(FlashProgressDialog)

    def retranslateUi(self, FlashProgressDialog):
        _translate = QtCore.QCoreApplication.translate
        FlashProgressDialog.setWindowTitle(_translate("FlashProgressDialog", "DSMCP - Writing progress"))
        self.labelIcon.setText(_translate("FlashProgressDialog", "TextLabel"))
        self.labelText.setText(_translate("FlashProgressDialog", "TextLabel"))
        self.labelRemainingTime.setText(_translate("FlashProgressDialog", "TextLabel"))



'''

:author:  F. Voillat
:date: 2021-04-28 Creation
:copyright: Dassym SA 2021
'''
from os.path import os
import re as RE
import datetime as DT
import functools

import struct
from dapi2.common import dateToWord, versionToStr
import logging


MONTHS = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec') 

def wordxorbytes(buf, l):
    return functools.reduce(lambda x,y : x^y, struct.unpack('<'+'H'*l, buf[:2*l] + b'\x00'*(2*l-len(buf))))

def buf2date(buf):
    s = buf.decode('ASCII')
    return DT.date(int(s[7:]), MONTHS.index(s[:3])+1, int(s[4:6]))

def readString(f, length):
    s = ''
    c = b'*'
    i=0
    while i < length: 
        c = f.read(1)
        i+=1
        if ord(c) != 0:
            s += chr(c[0])
        else:
            break
    return s


class Firmware(object):
    
    @classmethod
    def newFromConfig(cls, config):
        obj = Firmware(config.filename)
        obj.name = config.name
        obj.version = config.version
        try:
            obj.date =  DT.datetime.strptime(config.date, '%b %d %Y')
        except:
            obj.date =  DT.datetime.strptime(config.date, '%Y-%m-%d')
        obj.descr = config.descr
        obj.customer = config.customer
        obj.tag = wordxorbytes(bytes(obj.name,encoding="ASCII") , 16)
        obj.board_id = config.board[:2].lower()+config.board[3:5] 
        obj.type = config.type 
        return obj
    
    @classmethod
    def newFromFile(cls, filename, descr=None, customer=None):
        obj = Firmware(filename)
        offset = 0x000
        
        with open(filename, 'rb') as f:
            f.seek(0x00)
            magic_number = f.read(6)
            obj.type = 'firm'
            if magic_number != b"DASSYM":
                f.seek(0x10)
                magic_number = f.read(6)
                if magic_number != b"DASSYM":
                    raise Exception( 'The file `{0!s}` is not a Dassym firmware!'.format(os.path.basename(filename)))
                offset = 0x2000
                obj.type = 'full' 
            
            f.seek(0x08+offset)
            board_name = readString(f,8)
            obj.target = board_name[:2].lower()+"-"+board_name[3:5] 
            f.seek(0x10+offset)
            obj.version = ( ord(f.read(1)), ord(f.read(1)) )
            f.seek(0x14+offset)
            obj.name = readString(f,32)
            obj.tag = wordxorbytes(bytes(obj.name,encoding="ASCII") , 16)
            f.seek(0x34+offset)
            obj.date = buf2date(f.read(11))
            m = RE.match('^(\w+)-',obj.name)
            if m is not None:
                obj.customer = m.group(1) 
        return obj
        
    def __init__(self, filename):
        self.filename = filename
        self.name = None
        self.tag = None
        self.version = None
        self.date = None
        self.descr = None
        self.customer = None
        self.type = None
        self.target = None
        
    def __eq__(self, other):
        assert other is None or isinstance(other, Firmware)
        return other is not None and self.id == other.id and self.version == other.version    
    
    def __lt__(self, other):
        assert isinstance(other, Firmware)
        return self.tag < other.tag or (self.tag == other.tag and self.version < other.version)     
    
    def __str__(self):
        try:  
            return self.target +":"+self.name+" V"+versionToStr(self.version)+" ("+self.date.isoformat()+" 0x{0:04x})".format(self.tag)
        except:
            super().__str__()
        
    def softId(self):
        if self.tag is not None:
            return '{0:04x}'.format(self.tag)
        else:
            return None
        
    def key(self):
        return self.name + str(100-self.version[0]) + str(100-self.version[1]) + str(0xffff-dateToWord(self.date))         
        

class Firmwares(object):
    
    def __init__(self, app):
        self.log = logging.getLogger(self.__class__.__name__)
        self._app = app
        self._items = []
        
    def __len__(self):
        return len(self._items)
    
    def __getitem__(self, index):
        return self._items[index]
     
    def __iter__(self):
        for item in self._items:
            yield item     
        
        
    def discover(self, firm_paths, board=None):
        
        def scanDir(path):
            r = RE.compile(r".*_firm\.bin",)
            for folder, subs, files in os.walk(path):
                for fname in files:
                    if fname[0] == '.' or r.match(fname) is None:
                        continue  
                    #self.log.debug(fname)
                    firmware = Firmware.newFromFile(os.path.join(folder,fname))
                    
                    if firmware.type == 'full':
                        #self.log.warning('Full firmware :'+fname)
                        continue
                    if firmware.customer is None and not self._app.config.dev_mode:
                        self.log.warning('Invalid firmware :'+fname)
                        continue
                    
                    if board is not None and firmware.target != board.name:
                        continue
                    self._items.append(firmware)
                    self.log.debug(str(firmware))
                
                for pname in subs:
                    if pname[0]== '.':
                        continue
                    scanDir(os.path.join(folder,pname)) 
        
        for path in firm_paths:
            scanDir(path)
            
    def findLastFirmwareVersion(self, tag): 
        l = sorted([x for x in self._items if x.tag==tag])
        return l[-1] 
            
    def get(self, board_class, tag, version=None, date=None):
        trg = board_class.getName().lower()
        for firm in self:
            if firm.target == trg \
                    and tag == firm.tag \
                    and ((version is None) or version == firm.version) \
                    and((date is None) or firm.date == date) :
                return firm
        return None
    
    def find(self, board_class, tag=None):
        for firm in self:
            if firm.target == board_class.getName().lower() and (tag is None or tag == firm.tag):
                yield firm
            
    def isLast(self, firmware):
        return firmware is self.findLastFirmwareVersion(firmware.tag)        

        


'''

:author: fv
:date: Created on 5 mai 2021
'''
from PyQt5.Qt import QDialog, QIcon
from .ui_flashprogressdiaog import Ui_FlashProgressDialog
import logging
import time
from app.base import BaseApp

class FlashProgressDialog(QDialog, Ui_FlashProgressDialog):
    
    def __init__(self, parent, firmware):
        QDialog.__init__(self, parent)
        self.app = parent.app
        self.firmware = firmware
        self.log = logging.getLogger(self.__class__.__name__)
        self.log.debug( 'Initialize' )
        self.setupUi(self)
        self.t0 = time.time()
        self.labelIcon.setPixmap( QIcon(':/img/upload-firmware.svg').pixmap(128,128) )
        self.labelText.setText(BaseApp.tr('Wrinting of `{f!s}` in progress...'.format(f=self.firmware)))
    

    def progress(self, i, n):
        p = i/n
        self.progressBar.setValue(int(p*100))
        if p >= 0.05:
            t = time.time() 
            d = t - self.t0
            r = d*((1/p) - 1)  
            self.labelRemainingTime.setText(BaseApp.tr('Remaining time: {0:.0f}s'.format(r)))
        else:
            self.labelRemainingTime.setText('')
            
        self.app.processEvents()


'''

:author: fv
:date: Created on 19 mars 2021
'''

import dapi2



TRACE_INGOING_DAPI_FMT    = '{time:9.3f} | <-- | {buf:30s} | {msg.code!s}:{regs:s}'
TRACE_OUTGOING_DAPI_FMT   = '{time:9.3f} | --> | {buf:30s} | {msg.code!s}:{regs:s}'


class Tracer(object):
    def __init__(self, app, dapi=None):
        self.app = app
        self.dapi = dapi
        self.stop = False
        
    def traceRaw(self, time, direction, buf, msg):
        if self.stop:return
        if direction == dapi2.DComTracingDirection.INGOING:
            print(dapi2.TRACE_INGOING_COLOR, end='')
            print(dapi2.TRACE_INGOING_RAW_FMT.format(time=time, buf=dapi2.buffer2str(buf), msg=msg)) #@UndefinedVariable
        else:
            print(dapi2.TRACE_OUTGOING_COLOR, end='')
            print(dapi2.TRACE_OUTGOING_RAW_FMT.format(time=time, buf=dapi2.buffer2str(buf), msg=msg)) #@UndefinedVariable
            
    def traceNormal(self, time, direction, buf, msg):
        if self.stop:return
        if direction == dapi2.DComTracingDirection.INGOING:
            if msg.isError():
                print(dapi2.TRACE_INERROR_COLOR, end='')
            else:
                print(dapi2.TRACE_INGOING_COLOR, end='')
            print(dapi2.TRACE_INGOING_NORMAL_FMT.format(time=time, buf=dapi2.buffer2str(buf), msg=msg)) #@UndefinedVariable
        else:
            if msg.isError():
                print(dapi2.TRACE_OUTERROR_COLOR, end='')
            else:
                print(dapi2.TRACE_OUTGOING_COLOR, end='')
            print(dapi2.TRACE_OUTGOING_NORMAL_FMT.format(time=time, buf=dapi2.buffer2str(buf), msg=msg)) #@UndefinedVariable
            
    def traceDapi(self, time, direction, buf, msg):
        if self.stop:return
        if msg.type() in (dapi2.MsgType.READ, dapi2.MsgType.WRITE):
            regs =  self.dapi.regs[msg.getAddr():msg.getAddr()+msg.getLength()//dapi2.REG_SIZE]
        elif msg.type() == dapi2.MsgType.COMMAND:
            return self.traceNormal(time, direction, buf, msg)
        else:
            return #self.traceNormal(time, direction, buf, msg)
        
        if direction == dapi2.DComTracingDirection.INGOING:
            if msg.isError():
                print(dapi2.TRACE_INERROR_COLOR, end='')
                regs = str(msg)
            else:
                print(dapi2.TRACE_INGOING_COLOR, end='')
                if isinstance(msg, (dapi2.WrittenRegMessage, dapi2.ValueRegMessage)) :
                    regs = ', '.join('{reg.name:s}=0x{v:04x} ({v:d})'.format(reg=r, v=msg.getRegValue(i), i=i) for i, r in enumerate(regs))
                else:
                    regs = '???'
            print(TRACE_INGOING_DAPI_FMT.format(time=time, buf=dapi2.buffer2str(buf), msg=msg, regs=regs), end='')
        else:
            if msg.isError():
                print(dapi2.TRACE_OUTERROR_COLOR, end='')
                regs = str(msg)
            else:
                print(dapi2.TRACE_OUTGOING_COLOR, end='')
                if isinstance(msg, dapi2.WriteRegMessage) :
                    regs = ', '.join('0x{v:04x}({v:d})=>{reg.name:s}'.format(reg=r, v=msg.getRegValue(i), i=i) for i, r in enumerate(regs))
                elif isinstance(msg, dapi2.ReadRegMessage):
                    regs = ', '.join('{reg.name:s}?'.format(reg=r) for r in regs)
                else:
                    regs = '???'
            print(TRACE_OUTGOING_DAPI_FMT.format(time=time, buf=dapi2.buffer2str(buf), msg=msg, regs=regs), end='')
        
        print(dapi2.TRACE_RESET_COLOR)
        



from .common import USR_DATA_DIR, DATA_DIR, FIRM_DIR

import lxml.etree as ET
from os.path import os

resource_paths = {
    'user' : USR_DATA_DIR,
    'data' : DATA_DIR,
    'img'   : DATA_DIR,
    'firm'   : FIRM_DIR,
    }
'''Search paths for resource'''

def getRessourcePath(rname):
    try:
        prefix, fname = rname.split(':',1)
        return os.path.join( resource_paths[prefix], fname  )
    except ValueError:
        return rname
    
def getRessourceDir(rtype):
    return resource_paths[rtype]

def ressourceExists(rname):
    return os.path.exists(getRessourcePath(rname))


class PrefixResolver(ET.Resolver):
    def __init__(self, prefix_dict={}):
        self.prefix_dict = prefix_dict  
        
    def resolve(self, url, pubid, context):
        for prefix, replace_path in self.prefix_dict.items():
            if prefix[-1] == ':':
                prefix [:-1]
            if url.startswith(prefix):
                path = os.path.join(replace_path, url[len(self.prefix):])
                return self.resolve_filename(os.path.join(replace_path, path),context)